# -*- coding: utf-8 -*-
import os

from coredotfinance.krx.core.process import get_dataframe
from coredotfinance.krx.core.classify import get_krx_instance
from coredotfinance.krx.core import fetch, column
from coredotfinance.krx.core import jsp_util

"""
data_reader는 data.krx.co.kr로 부터 데이터를 가져온다.
불러온 데이터는 Json 형태이며 key 값은 영문약자로 되어 있다.
영문약자를 한글로 바꾸어 주기위해 data.krx.co.kr에서 .jsp 퍄일을 가져온다.
.jsp 파일의 url을 얻기위해서는 'MDCSTAT' 으로 시작하는 쿼리 단어를 얻어야 하고
그 단어는 post_params의 'bld' 값에 들어 있다.
"""


def data_reader(
    code, symbol=None, start=None, end=None, date=None, **kwargs
):
    """
    data.krx.co.kr 에서 데이터를 읽어 온다.

    Parameters
    ----------
    code : str
        수행하고자 하는 krx의 기능번호.
        https://data.krx.co.kr에서 이용하고자 하는 기능에 이용번호를 입력한다.
    symbol : str, int
        조회하고자 하는 데이터의 종목코드.
        형태는 종목과 종류마다 다르다. 예) 삼성전자 : '005930', ARIRANG 200 : '152100'
    start : str
        조회하고자 하는 데이터의 시작일.
        형태는 YYYYMMDD가 되어야 한다. 예) 20210601
    end : str
        조회하고자 하는 데이터의 종료일.
        형태는 YYYYMMDD가 되어야 한다. 예) 20210601
    date : str
        조회하고자 하는 데이터의 조회일.
        형태는 YYYYMMDD가 되어야 한다. 예) 20210601

    Warnings
    --------
    시작일, 종료일, 조회일이 공휴일일 경우에는 해당일의 데이터가 없다.
    예) data_reader("12003", symbol=symbol, start='202106019', end='20210620', kind=kind)을 호출하면 에러가 뜬다.


    Returns
    -------
    pd.DataFrame
    """

    if not isinstance(code, str):
        raise ValueError(f"code has to be {str} but got {type(code)}")
    krx_instance = get_krx_instance(code, symbol=symbol, start=start, end=end, date=date, **kwargs)
    post_params = krx_instance.get_requested_data()
    if symbol:
        symbol_name = krx_instance.data_nm
        print(symbol_name)

    mdcstat = _parse_mdcstat(post_params)
    jsp_soup = jsp_util.get_jsp_soup(mdcstat)

    valid_post_params = fetch.convert_vaild_post_params(jsp_soup, post_params)
    krx_data = fetch.get_krx_data(valid_post_params)

    korean_columns = column.get_korean_columns(jsp_soup, mdcstat)

    return get_dataframe(krx_data, korean_columns)


def _parse_mdcstat(post_params):
    """
    parses mdcstat from bld in post_params.

    Parameters
    ----------
    post_params: dict

    Examples :
        {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT01701",
            "tboxisuCd_finder_stkisu0_2": "060310/3S",
            "isuCd": "KR7060310000",
            "isuCd2": "060310",
            "codeNmisuCd_finder_stkisu0_2": "3S",
            "param1isuCd_finder_stkisu0_2": "STK",
            "strtDd": "000040",
            "endDd": "20210401",
            "MIME Type": "application/x-www-form-urlencoded; charset=UTF-8",
            "csvxls_isNo": "false",
        }

    Returns
    -------
    In this case, MDCSTAT01701 will be returned
    """

    bld = post_params["bld"]
    mdcstat = bld.split("/")[-1]
    return mdcstat



# -*- coding: utf-8 -*-

import numpy as np
import pandas as pd

from coredotfinance.util import datetime_util
from coredotfinance.krx.api.data_reader import data_reader


def convert_stock_ticker2name(stock: str) -> str:
    """'종목코드'(6자리)를 입력하면 '종목명' 반환"""
    stock_list = get_stock_info().loc[:, ["종목코드", "종목명"]]
    return stock_list[stock_list["종목코드"] == stock]["종목명"].array[0]


def convert_stock_name2ticker(stock: str) -> str:
    """종목명'을 입력하면 '종목코드' 반환"""
    stock_list = get_stock_info().loc[:, ["종목코드", "종목명"]]
    return stock_list[stock_list["종목명"] == stock]["종목코드"].array[0]


def get_stock_info(date: str = datetime_util.get_date_today()) -> pd.DataFrame:
    """KRX(KOSPI ,KOSDAQ) 종목 정보(종목코드, 종목명, 시장구분, 업종명, 시가총액) 반환"""
    df_12025_kospi = data_reader("12025", division="KOSPI", date=date).loc[
        :, ["종목코드", "종목명", "시장구분", "업종명", "시가총액"]
    ]
    df_12025_kosdaq = data_reader("12025", division="KOSDAQ", date=date).loc[
        :, ["종목코드", "종목명", "시장구분", "업종명", "시가총액"]
    ]
    df_12025 = (
        pd.concat([df_12025_kospi, df_12025_kosdaq])
        .astype({"시장구분": "category", "시가총액": "int64"})
        .sort_values(by=["시가총액"], ascending=False)
        .reset_index(drop=True)
    )
    return df_12025


def get_stock_pack(
    stock: str, start: str = datetime_util.get_date_past_days_ago(), end: str = datetime_util.get_date_today()
) -> pd.DataFrame:
    """주어진 기간의 일자별 개별종목의 정보들을 합쳐 하나의 데이터프레임으로 가져온다.
    Parameters
    ----------
    stock : str
        종목번호(ticker) 또는 종목명
    start : str, default : 오늘 날짜의 60일 전 날짜
        데이터 검색 시작 일자 (예: '20210324')
    end : str, default : 오늘 날짜
        데이터 검색 끝 일자 (예: '20210407')
    Returns
    -------
    pd.DataFrame
        KRX 정보데이터시스템의 개별종목 정보들을 합쳐놓은 DataFrame 반환
    """
    stock_list = get_stock_info().loc[:, ["종목코드", "종목명"]]
    if stock in stock_list["종목코드"].array:
        item = convert_stock_ticker2name(stock)
        item_code = stock
    elif stock in stock_list["종목명"].array:
        item = stock
        item_code = convert_stock_name2ticker(stock)
    else:
        raise Exception(f"Not in stock list : {stock}")

    # [12003] 개별종목 시세 추이
    df_12003 = data_reader("12003", symbol=item, start=start, end=end)
    # [12021] PER/PBR/배당수익률(개별종목)
    df_12021 = data_reader("12021", search_type="개별추이", symbol=item, start=start, end=end)
    # [12023] 외국인보유량(개별종목)
    df_12023 = data_reader("12023", search_type="개별추이", symbol=item, start=start, end=end)
    # [12009] 투자자별 거래실적(개별종목)
    df_12009 = get_df_12009(item=item, start=start, end=end)

    print("< 일자별 개별종목 종합정보 조회 >")
    print(f"종목명: {item} // 종목코드: {item_code} // 조회기간: {start}~{end}")
    df = pd.concat([df_12003, df_12021, df_12023, df_12009], axis="columns")
    df = df.loc[:, ~df.columns.duplicated()].sort_index(  # 중복되는 Columns 제외
        ascending=False
    )  # 최신 날짜가 위로 올라오도록 정렬
    return df


def get_df_12009(
    item, start: str = datetime_util.get_date_past_days_ago(), end: str = datetime_util.get_date_today()
) -> pd.DataFrame:
    """[12009] 투자자별 거래실적(개별종목) 조회 기능에서 2개 표(거래량-순매수, 거래대금-순매수)를 하나의 DataFrame으로 합쳐서 반환
    Parameters
    ----------
    item : str
        pack() 함수의 parameter인 stock이 종목명일 때 종목명을 가져옴
    item_code : str
        pack() 함수의 parameter인 stock이 종목코드일 때 종목코드를 가져옴
    start : str
        pack() 함수의 parameter인 start 일자를 가져옴
    end : str
        pack() 함수의 parameter인 end 일자를 가져옴
    Returns
    -------
    pd.DataFrame
        개별종목의 투자자별 거래실적 상세 항목을 모두 포함한 DataFrame 반환
    """
    trdvolval_list = ["거래량", "거래대금"]  # trdVolVal {1: '거래량', 2: '거래대금'}
    askbid_list = ["순매수"]  # askBid {1: '매도', 2: '매수', 3: '순매수'}
    df_list = []
    for trdvolval in trdvolval_list:
        for askbid in askbid_list:
            df_temp = data_reader(
                "12009",
                symbol=item,
                start=start,
                end=end,
                search_type="일별추이",
                trade_index=trdvolval,
                trade_check=askbid,
            )
            df_temp = df_temp.add_prefix(  # 매 번 반복되는 종목명 column 제거
                f"{trdvolval}_"
            )
            df_temp.columns = df_temp.columns.str.replace(" 합계", "")
            df_list.append(df_temp)
    df = pd.concat(df_list, axis="columns")
    return df


def get_adjusted_price(df: pd.DataFrame, col: str, inplace: bool = False) -> pd.Series:
    """
    DataFrame의 최근 일자의 상장주식수와 동일하도록 다른 날짜의 상장주식수를 수정하여 각 날짜별 수정 가격 Column을 반환
    Parameters
    ----------
    df : DataFrame
        수정 가격을 적용할 DataFrame
    col : str
        수정 가격을 적용할 원본 가격 Column 이름 ('종가', '시가', '고가', '저가')
    inplace : boolean, default = False
        원본 DataFrame에 '수정'이라는 이름을 앞에 붙인 Column을 추가하는 옵션
    Returns
    -------
    Series
        수정 가격이 적용된 Column(Series)
    Examples
    --------
    adjusted_price = get_adjusted_price(df, '종가')
    or
    adjusted_price = get_adjusted_price(df, '거래량', inplace=True)
    df
    """
    if col in ["종가", "시가", "고가", "저가", "거래량"]:
        latest_stocks = df["상장주식수"][df.index == df.index.max()].array[
            0
        ]  # DataFrame의 최근 일자의 상장주식수
        adjusted_price = (df[col] * (df["상장주식수"] / latest_stocks)).astype(
            np.int32
        )  # 수정 가격
        if col in ["거래량"]:
            adjusted_price = (df[col] / (df["상장주식수"] / latest_stocks)).astype(
                np.int32
            )  # 수정 거래량
        if inplace:
            df[
                f"수정{col}"
            ] = adjusted_price  # 원본 DataFrame에 '수정'이라는 이름을 앞에 붙인 Column을 추가 (기본값=False)
        return adjusted_price
    else:
        print("올바른 Column을 선택해주세요. (선택가능 Column: '종가', '시가', '고가', '저가', '거래량'")



index_code_list_stock = ["11001", "11002", "11003", "11004", "11005", "11006", "11007"]
index_code_list_bond = ["11008", "11009"]
index_code_list_derivation = ["11010", "11011", "11012", "11013", "11014"]

index_code_list = [str(code) for code in range(11001, 11014)]

stock_code_list = [str(code) for code in range(12001, 12029)]

product_code_list = [str(code) for code in range(13101, 13118)] + [str(code) for code in range(13201, 13217)] + [str(code) for code in range(13301, 13313)]


bond_code_list_price = ["14001", "14002"]
bond_code_list_info = ["14003", "14004"]
bond_code_list_trade = ["14005", "14006", "14007", "14008"]
bond_code_list_detail = [str(code) for code in range(14009, 14028)]

derivative_code_list_price = ["15001", "15002", "15003"]
derivative_code_list_info = ["15004", "15005"]
derivative_code_list_trade = ["15006", "15007", "15008", "15009"]
derivative_code_list_detail = [str(code) for code in range(15010, 15017)]

commodity_code_list_oil = [str(code) for code in range(16101, 16106)]
commodity_code_list_gold = [str(code) for code in range(16201, 16208)]
commodity_code_list_carbonemission = [str(code) for code in range(16301, 16305)]

oversees_code_list_euro = [str(code) for code in range(17101, 17109)]



import os
import requests

import pandas as pd
import numpy as np

__url = 'http://13.124.198.252:8080/'


def read(symbol, start, end, kind, resource, api_key, **kwargs):
    """
    api를 사용해서 해당 종목의 데이터를 받는

    Parameters
    ----------
    symbol : str
        조회하고자 하는 데이터의 종목코드.
        형태는 종목과 종류마다 다르다. 예) 삼성전자 : '005930', ARIRANG 200 : '152100'
    start : str
        조회하고자 하는 데이터의 시작일.
        형태는 YYYY-MM-DD가 되어야 한다. 예) 2021-06-01
    end : str
        조회하고자 하는 데이터의 종료일.
        형태는 YYYY-MM-DD가 되어야 한다. 예) 2021-06-01
    kind : str, default "stock"
        조회하고자 하는 데이터의 종류.
    api : bool, default False
        api_key가 설정되어 있지 않으면서 api가 True면 error가 발생한다.
        api 이용은 주식 가격만 가능하다.
    resource : str
        url 주소를 완성하기 위한 것으로 데이터의 출처값을 있다.
    api_key : str
        api key

    Returns
    -------
    pd.DataFrame

    """
    whole = kwargs.get('whole', False)
    url = os.path.join(__url, resource, kind, 'read', f'?symbol={symbol}&apikey={api_key}')
    response = requests.get(url).json()
    if response.get('msg', False):
        return response

    data = response['data']
    df = pd.DataFrame(data)
    df.index = df['날짜']
    df.index.name = ''
    transformed = df.drop(['종목명', '시장구분', '소속부', '날짜'], axis='columns')

    if not whole:
        transformed = transformed[np.logical_and(transformed.index >= start, transformed.index <= end)]

    return transformed


def read_date(date, kind, resource, api_key):
    """
    api를 사용해서 해당 일자의 전 종목 데이터를 받는다.

    Parameters
    ----------
    date : str
        조회하고자 하는 데이터의 조회일.
        형태는 YYYY-MM-DD가 되어야 한다. 예) 2021-06-01
    kind : str, default "stock"
        조회하고자 하는 데이터의 종류.
    resource : str
        url 주소를 완성하기 위한 것으로 데이터의 출처값을 있다.
    api_key : str
        api key

    Returns
    -------
    pd.DateFrame

    """

    url = os.path.join(__url, resource, kind, 'readall', f'?date={date}&apikey={api_key}')
    response = requests.get(url).json()
    if response.get('msg', False):
        return response

    data = response['data']
    data_list = []
    length = len(data['종목코드'])
    for i in range(length):
        data_dict = {}
        for key in data:
            data_dict[key] = data[key][str(i)]
        data_list.append(data_dict)

    return pd.DataFrame(data_list)



# -*- coding: utf-8 -*-
import pandas as pd
import numpy as np  # 'np.int32' 로 사용된다.
from datetime import datetime

second_column_map = {
    "BND_CLSS_NM1": "구분1",  # <- row map
    "BND_CLSS_NM2": "구분2",  # <- row map
}

no_display_columns = [
    "IND_TP_CD",
    "IDX_IND_CD",
    "MKT_ID",
    "CONV_OBJ_TP_CD",
    "ISU_ABBRV_STR",
    "ETF_ISU_CD",
    "BND_CLSS_CD",
    "NUSUAL_ISU_COND_CONTN",
    "KRW_FLUC_TP_CD",
    "OZ_FLUC_TP_CD",
    "FLUC_TP",
    "ISU_ABBRV",
    "SUB_IDX_IND_NM",
    "ISU_CD",
]


def get_dataframe(krx_data, column_map):
    _check_data_validation(krx_data)
    column_map.update(second_column_map)
    data = _apply_column_map(krx_data, column_map)
    data = _date_to_index(data)
    column_data = [column.split("//") for column in data.columns]
    columns_depth = max([len(c) for c in column_data])
    if not _single_column(columns_depth):
        column_data = _remove_same_named_column(column_data, columns_depth)
        columns = _multi_columnize(column_data, columns_depth)
        data.columns = columns
    data = _dataframe_astype(data)

    return data


def _check_data_validation(krx_data):
    if len(list(krx_data.values())[0]) == 0:
        raise Exception("No data, Check parameters")


def _apply_column_map(data_json, column_map):
    readable_column_list = []
    data_list = list(data_json.values())[0]

    for data in data_list:
        readable_column = {}
        for column, data_value in data.items():
            try:
                readable_column[column_map[column]] = data_value
            except:
                if column in no_display_columns or "TP_CD" in column:
                    continue
                else:
                    readable_column[column] = data_value
        readable_column_list.append(readable_column)
    return pd.json_normalize(readable_column_list)


def _date_to_index(data):
    if "일자" not in data.columns:
        # '일자' 열이 있는지 확인
        return data
    if len(data) != len(set(data["일자"].values)):
        # '일자' 가 중복되는 데이터인지 확인
        return data

    date_list = [datetime.strptime(date, "%Y/%m/%d") for date in data["일자"]]
    data.index = date_list
    return data.drop(["일자"], axis="columns")


def _single_column(columns_depth):
    # columns 가 single 인 경우
    if columns_depth == 1:
        return True


def _remove_same_named_column(column_data, columns_depth):
    # 같은 이름으로 multi columnize 되는 것을 방지
    for i in column_data:
        for _ in range(columns_depth - len(i)):
            i.append("")
    return column_data


def _multi_columnize(column_data, columns_depth):
    columns = []
    # column 만들기
    for i in range(1, columns_depth + 1):
        layer = []
        for column in column_data:
            layer.append(column[:i][-1])
        columns.append(layer)

    return columns


def _dataframe_astype(dataframe: pd.DataFrame):
    dataframe = _remove_punctuation(dataframe)
    column_data_type = _get_column_data_type(dataframe)
    dataframe = _0_to_empty_str(dataframe, column_data_type)
    return _data_type_as(dataframe, column_data_type)


def _remove_punctuation(dataframe: pd.DataFrame):
    dataframe.replace(',', '', regex=True, inplace=True)
    dataframe.replace('\-$', '0', regex=True, inplace=True)
    dataframe.replace('', '0', regex=True, inplace=True)

    return dataframe


def _get_column_data_type(dataframe: pd.DataFrame):
    """
    Returns
    -------
        when krx.read_date(date='2021-06-22')

        {'종목명': 'str',
         '시장구분': 'str',
         '소속부': 'str',
         '종가': 'np.int64',
         '대비': 'np.int64',
         '등락률': 'float',
         '시가': 'np.int64',
         '고가': 'np.int64',
         '저가': 'np.int64',
         '거래량': 'np.int64',
         '거래대금': 'np.int64',
         '시가총액': 'np.int64',
         '상장주식수': 'np.int64'}

         또는

        when krx.read('152100', start = '2012-01-01', end='2012-02-01', kind='etf')

         {('종가', ''): 'np.int64',
         ('대비', ''): 'np.int64',
         ('등락률', ''): 'float',
         ('지표가치(IV)', ''): 'float',
         ('시가', ''): 'np.int64',
         ('고가', ''): 'np.int64',
         ('저가', ''): 'np.int64',
         ('거래량', ''): 'np.int64',
         ('거래대금', ''): 'np.int64',
         ('시가총액', ''): 'np.int64',
         ('지표가치총액', ''): 'np.int64',
         ('상장증권수', ''): 'np.int64',
         ('기초지수', '지수명'): 'str',
         ('기초지수', '종가'): 'float',
         ('기초지수', '대비'): 'float',
         ('기초지수', '등락률'): 'float'}
    """
    column_data_type = {}

    for column in dataframe.columns:
        if column in ['종목코드', ('종목코드', '')]:
            continue
        for data in dataframe[column]:
            if data == '' or data == '0':
                continue
            try:
                data = eval(data)
            except:
                # 문자가 들어있는 str은 eval이 작동하지 못한다. ex) 3S, KOSDAQ, 중견기업부
                pass
            data_type = type(data)

            if data_type is str:
                column_data_type[column] = 'str'
                break
            elif data_type is int:
                column_data_type[column] = 'np.int64'
                break
            elif data_type is float:
                column_data_type[column] = 'float'
                break

    return column_data_type


def _0_to_empty_str(dataframe: pd.DataFrame, column_data_type: dict):
    """
    데이터가 str인 column에 들어있는 0을 '' 로 바꾸어 준다.
    column_data_type 에서 value가 'str' 인 column 만 바꾸어 준다.
    """
    for column, datatype in column_data_type.items():
        if datatype == 'str':
            dataframe[column].replace('0', '', inplace=True)
    return dataframe


def _data_type_as(dataframe: pd.DataFrame, column_data_type: dict):
    """
    모든 데이터가 str으로 들어가 있는 dateframe을 column_data_type에 담겨있는 정보에 따라서
    데이터의 type을 바꾸어 준다.
    소수로 이루어져 있는 데이터는 str -> float
    정수로 이루어져 있는 데이터는 str -> np.int64

    Parameters
    ----------
    dataframe
    column_data_type

    Returns
    -------

    """
    for column, datatype in column_data_type.items():
        if datatype == 'str':
            continue
        try:
            dataframe = dataframe.astype({column: eval(datatype)})
        except ValueError:
            # 햔 column에 int와 float이 같이 들어있는 경우가 있다. 그때!! 바로!! 쓴다!!
            dataframe = dataframe.astype({column: float})
    return dataframe



from coredotfinance.krx.krx_website.info import Info


# testing for checking '&' having word, kt&g
def test_autocomplete_stock_1():
    assert Info(None, None, None).autocomplete("kt&g", "stock") == (
        "KT&G",
        "KR7033780008",
        "033780",
    )


# testing for checking word priority, 바이온
def test_autocomplete_stock_2():
    assert Info(None, None, None).autocomplete("바이온", "stock") == (
        "바이온",
        "KR7032980005",
        "032980",
    )


def test_autocomplete_index():
    assert Info(None, None, None).autocomplete("zhtmvl", "index") == ("코스피", "1", "001")


def test_autocomplete_ETF():
    assert Info(None, None, None).autocomplete("arirang alrnrsk", "ETF") == (
        "ARIRANG 미국나스닥기술주",
        "KR7287180004",
        "287180",
    )


def test_autocomplete_ETN():
    assert Info(None, None, None).autocomplete("qv 미국", "ETN") == (
        "QV 미국 IT TOP5 ETN(H)",
        "KRG552100411",
        "550041",
    )


def test_autocomplete_ELW():
    assert Info(None, None, None).autocomplete("37tkatjdwjswk", "ELW") == (
        "KBF937삼성전자콜",
        "KRA5811BKA77",
        "58F937",
    )


def test_autocomplete_dericative():
    assert Info(None, None, None).autocomplete("a", "derivative") == (
        "ARIRANG 고배당 F 202106",
        "KR41M2R60003",
        "1M2R6000",
    )


def test_autocomplete_publish():
    assert Info(None, None, None).autocomplete("a", "publish") == (
        "AJ네트웍스",
        "09557",
        "09557",
    )



import re
import datetime
import warnings

from coredotfinance.krx.api.data_reader import data_reader
from coredotfinance.binance import binance
from coredotfinance.database import krx_db


class KrxReader:
    """
    krx data를 읽어오는 인스턴스를 생성한다. 많은 양의 데이터를 짧은 시간안에
    불러오게 하면 krx에서 IP를 차단하기 때문에 많은 양의 데이터 읽어오기는
    api 기능을 사용하는 것을 권장한다.

    Parameters
    ----------
    api_key : str
        coredotfinance의 데이터베이스에서 데이터를 받아오기 위해서는
        api_key 설정이 필요하다. api 기능을 사용해서 IP 차단을 피할 수 있다.
    """

    not_service_api = ["etf", "etn", "elw", "per"]

    def __init__(
            self,
            api_key=None,
    ):
        self.api_key = api_key

    def _date_check(self, date):
        """
        date 가 None 이면 return 한다. None date 는 today 로 convert 되기 때문이다.
        """
        if date is None:
            return
        if re.match(r'[0-9]{4}-[0-1][0-9]-([0-2][0-9]|3[0-1])', date) is None:
            raise ValueError(f"date is supposed to be 'YYYY-MM-DD and proper date, but {date}")

    def _date_convert(self, date):
        """
        date 가 None 이면 today 를 return 한다.

        Parameters
        ----------

        date : str
            YYYY-MM-DD

        Returns
        --------
        str
            YYYYMMDD
        """
        today = str(datetime.datetime.now().date())
        if date is None:
            date = today
        return date.replace('-', '')

    def _kind_check(self, kind):
        expected_kind = ["stock", "etf", "etn", "elw", "per"]
        if kind not in expected_kind:
            raise ValueError(f"expected kind in {expected_kind}, but got {kind}")

    def _api_key_check(self, api):
        if self.api_key is None and api is not False:
            raise ValueError("api_key has to be set in order to use api")

    def read(
            self,
            symbol,
            *,
            start=None,
            end=None,
            kind="stock",
            api=False
    ):
        """
        해당 주식 가격 데이터를 시작일(start) 부터 종료일(end) 까지 읽어온다.

        Parameters
        ----------
        symbol : str
            조회하고자 하는 데이터의 종목코드.
            형태는 종목과 종류마다 다르다. 예) 삼성전자 : '005930', ARIRANG 200 : '152100'
        start : str
            조회하고자 하는 데이터의 시작일.
            형태는 YYYY-MM-DD가 되어야 한다. 예) 2021-06-01
        end : str
            조회하고자 하는 데이터의 종료일.
            형태는 YYYY-MM-DD가 되어야 한다. 예) 2021-06-01
        kind : str, default "stock"
            조회하고자 하는 데이터의 종류.
            데이터의 종류 - krx : ["stock", "etf", "etn", "elw", "per"]
        api : bool, default False
            api_key가 설정되어 있지 않으면서 api가 True면 error가 발생한다.
            api 이용은 주식 가격만 가능하다.

        Returns
        -------
        pd.DataFrame
            data
        """

        if start is None or end is None:
            warnings.warn("""start or end is None. 
                          It would lead an error because datetime.datetime.now() is default 
                          and it could be holiday when stock marker was not held 
                          or before stock marker is opened""")

        if api and kind in KrxReader.not_service_api:
            raise ValueError(f"{kind} does not service api yet.")

        self._date_check(start)
        self._date_check(end)
        start_8_digit = self._date_convert(start)
        end_8_digit = self._date_convert(end)
        self._kind_check(kind)
        self._api_key_check(api)

        if start_8_digit > end_8_digit:
            raise ValueError(f"start has to be earlier than end, but {start}, {end}")

        if api:
            return krx_db.read(symbol, start, end, kind=kind, resource='krx', api_key=self.api_key)

        if kind == "stock":
            return data_reader("12003", symbol=symbol, start=start_8_digit, end=end_8_digit, kind=kind)
        elif kind == "per":
            return data_reader("12021", symbol=symbol, start=start_8_digit, end=end_8_digit, kind=kind, search_type="개별추이")
        elif kind == "etf":
            return data_reader("13103", symbol=symbol, start=start_8_digit, end=end_8_digit, kind=kind)
        elif kind == "etn":
            return data_reader("13203", symbol=symbol, start=start_8_digit, end=end_8_digit, kind=kind)
        elif kind == "elw":
            return data_reader('13302', symbol=symbol, start=start_8_digit, end=end_8_digit, kind=kind)
        else:
            raise ValueError(f"Check {kind} is not in the list of expected_kind")

    def read_all(
            self,
            symbol,
            *,
            kind="stock",
            api=False
    ):
        """
        전기간의 해당 주식 가격 데이터를 읽어온다.

        Parameters
        ----------
        symbol : str
            조회하고자 하는 데이터의 종목코드.
            형태는 종목과 종류마다 다르다. 예) 삼성전자 : '005930', ARIRANG 200 : '152100'
        start : str
            조회하고자 하는 데이터의 시작일.
            형태는 YYYY-MM-DD가 되어야 한다. 예) 2021-06-01
        end : str
            조회하고자 하는 데이터의 종료일.
            형태는 YYYY-MM-DD가 되어야 한다. 예) 2021-06-01
        kind : str, default "stock"
            조회하고자 하는 데이터의 종류.
            krx : ["stock", "etf", "etn", "elw", "per"]
        api : bool, default False
            api_key가 설정되어 있지 않으면서 api가 True면 error가 발생한다.
            api 이용은 주식가격만 가능하다.

        Returns
        -------
        pd.DataFrame
            data
        """
        return self.read(symbol, start='1900-01-01', end='2030-01-01', kind=kind, api=api)

    def read_date(self, date=None, *, kind='stock', api=False):
        """
        해당 일자의 전 종목 주식 데이터를 불러온다.

        Parameters
        ----------
        date : str
            조회하고자 하는 데이터의 조회일.
            형태는 YYYY-MM-DD가 되어야 한다. 예) 2021-06-01
        kind : str, default "stock"
            조회하고자 하는 데이터의 종류.
            데이터의 종류 - krx : ["stock", "etf", "etn", "elw", "per"]
        api : bool, default False
           api_key가 설정되어 있지 않으면서 api가 True면 error가 발생한다.

        Returns
        -------
        pd.DataFrame
            data
        """

        if api and kind in KrxReader.not_service_api:
            raise ValueError(f"{kind} does not service api yet.")

        if date is None:
            warnings.warn("""date is None.
                          It would lead an error because datetime.datetime.now() is default 
                          and it could be holiday when stock marker was not held 
                          or before stock marker is opened""")

        self._date_check(date)
        date_8_digit = self._date_convert(date)
        self._kind_check(kind)
        self._api_key_check(api)

        if api:
            return krx_db.read_date(date, kind=kind, resource='krx', api_key=self.api_key)

        if kind == "stock":
            return data_reader("12001", date=date_8_digit, kind=kind)
        elif kind == "per":
            df = data_reader("12021", search_type="전종목", market='전체', date=date_8_digit)
            # 12021 기능 호출시 종목명 <em class ="up"></em> 가 붙어서 나오는 문제를 해결하기 위함
            df.replace(' <em class ="up"></em>', '', regex=True, inplace=True)
            return df
        elif kind == "etf":
            return data_reader("13101", date=date_8_digit, kind=kind)
        elif kind == "etn":
            return data_reader("13201", date=date_8_digit, kind=kind)
        elif kind == "elw":
            return data_reader('13301', date=date_8_digit, kind=kind)
        else:
            raise ValueError(f"Check {kind} is not in the list of expected_kind")


class BinanceReader:
    """
        Returns DataFrame of crypto currency price data from binance.com
        Trying to get bulky data through many times of iteration leads IP blocking.
        So using api to get bulky data is highly recommended.
    """

    def __init__(
            self,
            api_key=None,
    ):
        self.api_key = api_key

    @property
    def symbols(self):
        return binance.get_symbols()

    def read(self, symbol, start, end, interval):
        """
        해당 암호화폐의 가격 데이터를 불러온다.

        Parameters
        ----------
        symbol : str, optional
            Binance Symbol
        interval : str, optional
            조회 간격 설정, by default "1d"
            (1m, 3m, 5m, 15m, 30m, 1h, 2h, 4h, 6h, 8h, 12h, 1d, 3d, 1w, 1M)
        start : str, optional
            조회 시작 날짜(YYYY-MM-DD), by default 최근 날짜
        end : str, optional
            조회 끝 날짜(YYYY-MM-DD), by default 최근 날짜

        Returns
        -------
        pd.DataFrame
            data
        """

        start = start.replace('-', '')
        end = end.replace('-', '')

        return binance.get_ohlcv(symbol=symbol, start=start, end=end, interval=interval)


# -*- coding: utf-8 -*-
from coredotfinance.krx.core.krx_website.info import Info


class Bond(Info):
    def __init__(self, code, start, end, day, item, code_to_function):
        super(Bond, self).__init__(start, end, day)
        self.get_requested_data = code_to_function[code]
        if code in ["14011", "14021", "14023"]:
            self.data_nm, self.data_cd, self.data_tp = self.autocomplete(
                item, "publish"
            )
        else:
            self.data_nm, self.data_cd, self.data_tp = self.autocomplete(item, "bond")


class ItemPrice(Bond):
    def __init__(self, code, start, end, day, item, **kwargs):
        code_to_function = {
            "14001": self.price_of_entire_item,
            "14002": self.price_trend_of_item,
        }
        super().__init__(code, start, end, day, item, code_to_function)
        self.market = kwargs.get("market", None)

    def price_of_entire_item(self):
        """전종목 시세 [14001]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT09801",
            "mktId": self.market,
            "trdDd": self.day,
        }
        return self.update_requested_data(data)

    def price_trend_of_item(self):
        """개별종목 시세 추이 [14002]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT09901",
            "mktId": self.market,
            "isuCd": self.data_cd,
            "tboxisuCdBox0_finder_bondisu0_2": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)


class ItemInfo(Bond):
    def __init__(self, code, start, end, day, item, **kwargs):

        code_to_function = {"14003": self.info_of_entire_item, "14004": ""}
        self.bond_type = kwargs.get("bond_type", None)
        super(ItemInfo, self).__init__(code, start, end, day, item, code_to_function)

    def info_of_entire_item(self):
        """전종목 기본정보 [14003]"""
        data = {"bld": "dbms/MDC/STAT/standard/MDCSTAT10001", "bndTpCd": self.bond_type}
        return self.update_requested_data(data)

    def entire_info_of_itme(self):
        """'개별종목 종합정보 [14004]"""
        """not now"""


class TradePerform(Bond):
    def __init__(self, code, start, end, day, item, **kwargs):
        code_to_function = {
            "14005": self.trade_performance_per_category,
            "14006": self.trade_performance_per_investor,
            "14007": self.trade_performance_of_bond_index_item,
            "14008": self.trade_performance_of_Repo,
        }
        super(TradePerform, self).__init__(
            code, start, end, day, item, code_to_function
        )
        self.market = kwargs.get("market", None)
        self.inquiry = kwargs.get("inquiry", None)

    def trade_performance_per_category(self):
        """종류별 거래실적 [14005]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT10201",
            "bndMktTpCd": self.market,
            "inqTpCd": self.inquiry,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def trade_performance_per_investor(self):
        """투자자별 거래실적 [14006]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT10301",
            "bndMktTpCd": self.market,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def trade_performance_of_bond_index_item(self):
        """국채지표종목 거래실적 [14007]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT10401",
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def trade_performance_of_Repo(self):
        """Repo 거래실적 [14008]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT10501",
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)


class Detail(Bond):
    def __init__(self, code, start, end, day, item, **kwargs):
        code_to_funciton = {
            "14009": self.price_assessment_trend_of_item,
            "14010": self.reported_price_trend_of_small_bond,
            "14011": self.search_of_public_bond,
            "14012": self.issue_info_of_public_bond,
            "14013": self.histoty_per_type_of_publication,
            "14014": self.history_of_publication_price_adjustment,
            "14015": self.prepayment,
            "14016": self.delisting_bond,
            "14017": self.rate_of_profit_of_bond_over_the_counter,
            "14018": self.rate_of_profit_of_index,
            "14019": self.strip_short_interest,
            "14020": self.substitution_price_of_bond,
            "14021": self.substitution_price_of_bond_per_issuer,
            "14022": self.substitution_price_of_bond_per_type,
            "14023": self.credit_per_issuer,
            "14024": self.publication_situation_per_credit,
            "14025": self.investment_index_of_convertible_bond,
            "14026": self.exercise_of_right_of_bond_about_stock,
            "14027": self.strike_price_of_bond_about_stock,
        }
        super().__init__(code, start, end, day, item, code_to_funciton)
        self.market = kwargs.get("market", None)
        self.inquiry = kwargs.get("inquiry", None)
        self.bond_type = kwargs.get("bond_type", None)
        self.search_type = kwargs.get("search_type", None)

    def price_assessment_trend_of_item(self):
        """개별종목 시가평가 추이 [14009]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT10601",
            "tboxisuCd_finder_bondisu0_0": f"{self.data_cd}/{self.data_nm}",
            "isuCd": self.data_cd,
            "codeNmisuCd_finder_bondisu0_0": self.data_nm,
            "param1isuCd_finder_bondisu0_0": 2,
            "basddTpCd": self.inquiry,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def reported_price_trend_of_small_bond(self):
        """소액채권 신고가격 추이 [14010]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT10701",
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def search_of_public_bond(self):
        """상장채권 상세검색 [14011]"""
        # 기능중 채권분류 및 상세검색은 추가하지 않았음
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT10801",
            "tboxisurCd_finder_bndordisu0_2": f"{self.data_tp}/{self.data_nm}",
            "isurCd": self.data_tp,
            "isurCd2": self.data_tp,
            "codeNmisurCd_finder_bndordisu0_2": self.data_nm,
            "bndTpCd": self.bond_type,
        }
        return self.update_requested_data(data)

    def issue_info_of_public_bond(self):
        """상장채권 발행정보 [14012]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT10901",
            "tboxisuCd_finder_bondisu0_2": f"{self.data_cd}/{self.data_nm}",
            "isuCd": self.data_cd,
            "isuCd2": self.data_cd,
            "codeNmisuCd_finder_bondisu0_2": self.data_nm,
        }
        return self.update_requested_data(data)

    def histoty_per_type_of_publication(self):
        """상장유형별 내역 [14013]"""
        if self.search_type == "추가":
            bld = "dbms/MDC/STAT/standard/MDCSTAT11002"
        elif self.search_type == "변경":
            bld = "dbms/MDC/STAT/standard/MDCSTAT11003"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT11001"

        data = {
            "bld": bld,
            "inqTpCd": self.search_type,
            "BndMktactTpCd": self.bond_type,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def history_of_publication_price_adjustment(self):
        """상장금액조정 내역 [14014]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT11101",
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def prepayment(self):
        """중도상환 [14015]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT11201",
            "bndMktactTpCd": self.inquiry,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def delisting_bond(self):
        """채권상장폐지 [14016]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT11301",
            "bndDelistRsnCd": self.inquiry,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def rate_of_profit_of_bond_over_the_counter(self):
        """장외 채권수익률 [14017]"""
        #### ExecuteForResourceBundle 의 url이 requests.get으로 안받아와진다!!
        #### http://data.krx.co.kr/comm/bldAttendant/executeForResourceBundle.cmd?baseName=krx.mdc.i18n.component&key=B160.bld&type=kospi
        if self.search_type == "개별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT11402"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT11401"
        output = {
            "국고채 1년": "3006",
            "국고채 2년": "3019",
            "국고채 3년": "3000",
            "국고채 5년": "3007",
            "국고채 10년": "3013",
            "국고채 20년": "3014",
            "국고채 30년": "3017",
            "국민주택 1종 5년": "3008",
            "회사채 AA-(무보증 3년)": "3009",
            "회사채 BBB- (무보증 3년)": "3010",
            "CD(91일)": "4000",
        }
        data = {
            "bld": bld,
            "inqTpCd": self.search_type,
            "trdDd": self.day,
            "bndKindTpCd": output[self.bond_type],
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def rate_of_profit_of_index(self):
        """지표 수익률 [14018]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT11501",
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def strip_short_interest(self):
        """스트립 단기금리 [14019]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT11601",
            "inqTpCd": self.inquiry,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def substitution_price_of_bond(self):
        """채권 대용가 [14020]"""
        if self.search_type == "개별추이":
            bld = None  # jsGrid_dict를 찾지 못함...
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT11701"
        data = {
            "bld": bld,
            "tboxisuCd_finder_bondisu0_10": f"{self.data_cd}/{self.data_nm}",
            "isuCd": self.data_cd,
            "isuCd2": self.data_cd,
            "codeNmisuCd_finder_bondisu0_10": self.data_nm,
            "param1isuCd_finder_bondisu0_10": 2,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def substitution_price_of_bond_per_issuer(self):
        """발행기관별 채권 대용가 [14021]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT11801",
            "tboxisurCd_finder_bndordisu0_1": f"{self.data_tp}/{self.data_nm}",
            "isurCd": self.data_tp,
            "isurCd2": self.data_tp,
            "codeNmisurCd_finder_bndordisu0_1": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def substitution_price_of_bond_per_type(self):
        """유형별 채권 대용가 [14022]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT11901",
            "bndTpCd": self.bond_type,
            "bndClssCd": self.inquiry,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def credit_per_issuer(self):
        """발행기관별 신용등급 [14023]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT12001",
            "tboxisurCd_finder_bndordisu0_14": f"{self.data_tp}/{self.data_nm}",
            "isurCd": self.data_tp,
            "isurCd2": self.data_tp,
            "codeNmisurCd_finder_bndordisu0_14": self.data_nm,
            "creditValuInstCd": self.inquiry,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def publication_situation_per_credit(self):
        """신용등급별 상장현황 [14024]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT12101",
            "creditValuInstCd": self.inquiry,
            "trdDd": self.day,
        }
        return self.update_requested_data(data)

    def investment_index_of_convertible_bond(self):
        """전환사채 투자지표 [14025]"""
        data = {"bld": "dbms/MDC/STAT/standard/MDCSTAT12201", "trdDd": self.day}
        return self.update_requested_data(data)

    def exercise_of_right_of_bond_about_stock(self):
        """주식관련채권 권리행사 [14026]"""
        # 'bld': 'dbms/MDC/STAT/standard/MDCSTAT12301' 을 데이터로 사용하면 전체 데이터가 출력된다.
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT12302",
            "tboxisuCd_finder_bondisu0_17": f"{self.data_cd}/{self.data_nm}",
            "isuCd": self.data_cd,
            "isuCd2": self.data_cd,
            "param1isuCd_finder_bondisu0_17": 2,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def strike_price_of_bond_about_stock(self):
        """주식관련채권 행사가액 [14027]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT12401",
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)



import pytest

from coredotfinance.krx.utils import classifier, start_end_validation, week_day


def test_classifier():
    pass


@pytest.mark.parametrize(
    "good_ex",
    [
        (None, None),
        (20210420, None),  # from start to 60 days later
        (20210420, 20210423),  # 4 biz days
        (20210420, 20210420),  # a biz day
    ],
)
def test_start_end_validation_good(good_ex):
    assert start_end_validation(*good_ex)


@pytest.mark.parametrize(
    "bad_ex",
    [
        (2021042, 20210423),  # wrong start len is under 8-digit
        (20210420, 2021042),  # wrong end len is under 8-digit
        (20210423, 20210420),  # start > end
        (20210333, 20210413),  # day is out of range of month
    ],
)
def test_start_end_validation_bad(bad_ex):
    try:
        if start_end_validation(*bad_ex):
            raise Exception("test Fail")
    except Exception:
        pass  # test passed


def test_week_day():
    assert 4 == week_day("20210423")


@pytest.mark.parametrize(
    "item_name",
    [
        ("3s",),
        ("AJ네트웍스",),
        ("ARIRANG 200",),
        ("KB 레버리지 S&P 500 선물 ETN(H)",),
        ("KBF937삼성전자콜", "elw"),
    ],
)
def test_classifier_item_name(item_name):
    assert "item name" == classifier(*item_name)


@pytest.mark.parametrize(
    "item_code", [("211270",), ("00104K",), ("58F937", "elw"), ("58FK37", "elw")]
)
def test_classifier_item_code(item_code):
    assert "item code" == classifier(*item_code)



from datetime import datetime, timezone

from coredotfinance._utils import (
    _convert_date2timestamp_sec,
    _convert_timestamp2datetime_list,
    _get_date_past_days_ago,
    _get_date_today,
)
from freezegun import freeze_time


@freeze_time("20210420")
def test_get_today():
    assert _get_date_today() == "20210420"


@freeze_time("20210420")
def test_get_past_days_ago():
    assert _get_date_past_days_ago(10) == "20210410"


def test_convert_date2timestamp():
    assert _convert_date2timestamp_sec("20200420") == 1587340800


def test_convert_timestamp2datetime_list():
    assert _convert_timestamp2datetime_list([1587308400]) == [datetime(2020, 4, 20)]



# -*- coding: utf-8 -*-
import requests
from bs4 import BeautifulSoup as bs
from coredotfinance.krx.core.krx_website.info import Info


class Index(Info):
    def __init__(self, code, code_to_function, start, end, day):
        self.get_requested_data = code_to_function[code]
        super(Index, self).__init__(start, end, day)


class StockIndex(Index):
    def __init__(self, code, start, end, day, division, item, **kwargs):
        """주가지수
        :param code: 항목 고유 번호
        :param start: 시작일
        :param end: 종료일
        :param day: 조회일자
        :param division:
        :param item:
        """
        code_to_function = {
            "11001": self.price_of_entire_index,
            "11002": self.fluc_of_entire_index,
            "11003": self.trend_of_index_price,
            "11004": self.info_of_entire_index,
            "11005": "Not now",
            "11006": self.stocks_of_index,
            "11007": self.per_pbr_dividend_of_index,
        }

        super().__init__(code, code_to_function, start, end, day)
        self.data_nm, self.data_cd, self.data_tp = self.autocomplete(item, "index")
        self.division = "KRX" if division is None else division.upper()
        self.search_type = kwargs.get("search_type", "전체지수")

    def price_of_entire_index(self):
        """전체 지수 시세 [11001]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT00101",
            "idxIndMidclssCd": self.division,
            "trdDd": self.day,
        }
        return self.update_requested_data(data)

    def fluc_of_entire_index(self):
        """전체 지수 변동률 [11002]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT00201",
            "idxIndMidclssCd": self.division,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def trend_of_index_price(self):
        """개별 지수 시세추이 [11003]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT00301",
            "tboxindIdx_finder_equidx0_0": self.data_nm,
            "indIdx": self.data_cd,
            "indIdx2": self.data_tp,
            "codeNmindIdx_finder_equidx0_0": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def info_of_entire_index(self):
        """전체 지수 기본 정보 [11004]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT00401",
            "idxIndMidclssCd": self.division,
        }
        return self.update_requested_data(data)

    def info_of_index(self):
        """개별 지수 종합 정보 [11005]"""
        pass

    def stocks_of_index(self):
        """지구 구성 종목 [11006]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT00601",
            "tboxindIdx_finder_equidx0_2": self.data_nm,
            "indIdx": self.data_cd,
            "indIdx2": self.data_tp,
            "codeNmindIdx_finder_equidx0_2": self.data_nm,
            "trdDd": self.day,
        }
        return self.update_requested_data(data)

    def per_pbr_dividend_of_index(self):
        """개별지수 PER/PBR/배당수익률 [11007]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT00702",
            "searchType": self.search_type,
            "indTpCd": self.data_cd,
            "indTpCd2": self.data_tp,
            "codeNmindTpCd_finder_equidx0_3": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
            "idxIndMidclssCd": self.division,
            "trdDd": self.day,
        }
        return self.update_requested_data(data)


class BondIndex(Index):
    def __init__(self, code, start, end, day, division):
        """
        :param code:
        :param start:
        :param end:
        :param day:
        :param division:
        """

        code_to_function = {
            "11008": self.price_of_entire_index,
            "11009": self.trend_of_index,
        }
        super().__init__(code, code_to_function, start, end, day)

        self.division = "KRX채권" if division is None else division.upper()

    def price_of_entire_index(self):
        """전체 지수 시세 [11008]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT00801",
            "trdDd": self.day,
        }
        return self.update_requested_data(data)

    def trend_of_index(self):
        """개별 지수 시세 추이 [11009]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT00901",
            "indTp": self.division,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)


class DerivationIndex(Index):
    def __init__(self, code, start, end, day, division, item):
        code_to_function = {
            "11010": self.price_of_entire_index,
            "11011": self.fluc_of_entire_index,
            "11012": self.trend_of_index,
            "11013": self.info_of_entire_index,
            "11014": "Not now",
        }
        super().__init__(code, code_to_function, start, end, day)
        if item:
            self.data_nm, self.data_cd, self.data_tp = self.autocomplete(item)
        self.division = "선물지수" if division is None else division.upper()
        self.update_requested_data = code_to_function[code]

    def autocomplete(self, item):
        if item is None:
            item = "코스피 200 선물지수"
        index_autocomplete_url = "http://data.krx.co.kr/comm/finder/autocomplete.jspx?contextName=finder_drvetcidx&value={value}&viewCount=5&bldPath=%2Fdbms%2Fcomm%2Ffinder%2Ffinder_drvetcidx_autocomplete"
        response = requests.get(index_autocomplete_url.format(value=item))
        soup = bs(response.content, "html.parser").li

        if soup is None:
            raise AttributeError(f"{item} is Wrong name as an index name")

        data_nm = soup.attrs["data-nm"]
        data_cd = soup.attrs["data-cd"]
        data_tp = soup.attrs["data-tp"]

        return data_nm, data_cd, data_tp

    def price_of_entire_index(self):
        """전체 지수 시세 [11010]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT01001",
            "clssCd": self.division,
            "trdDd": self.day,
        }
        return self.update_requested_data(data)

    def fluc_of_entire_index(self):
        """전체 지수 등락률 [11011]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT01101",
            "clssCd": self.division,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def trend_of_index(self):
        """개별 지수 시세 추이 [11012]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT01201",
            "indTpCd": self.data_cd,
            "idxIndCd": self.data_tp,
            "tboxidxCd_finder_drvetcidx0_0": self.data_nm,
            "idxCd": self.data_cd,
            "idxCd2": self.data_tp,
            "codeNmidxCd_finder_drvetcidx0_0": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def info_of_entire_index(self):
        """전체 지수 기본 정보 [11013]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT01301",
            "idxTp": self.division,
        }
        return self.update_requested_data(data)



_cols_kor = {
    "symbol": "종목코드",
    "datetime": "일시",
    "open": "시가",
    "high": "고가",
    "low": "저가",
    "close": "종가",
    "volume": "거래량",
    "priceChange": "대비",
    "priceChangePercent": "등락률",
    "lastPrice": "종가",
    "openPrice": "시가",
    "highPrice": "고가",
    "lowPrice": "저가",
    "tradingValue": "거래대금",
    "bid_price": "매수호가",
    "bid_volume": "매수물량",
    "ask_price": "매도호가",
    "ask_volume": "매도물량",
    "adjopen": "수정시가",
    "adjhigh": "수정고가",
    "adjlow": "수정저가",
    "adjclose": "수정종가",
    "adjvolume": "수정거래량",
}


def rename_cols2kor(df):
    df = df.rename(columns=_cols_kor)
    return df






import json

from coredotfinance.krx.core import webio


def convert_vaild_post_params(jsp_soup, post_params):
    """
    post_params needs to be transformed in order to post krx data.
    User input as Korean word is converted into what krx server understands

    Parameters
    ----------
    jsp_soup : bs4.BeautifulSoup
        result of
    post_params : dict
        Examples :
            post_params : {
                "bld": "dbms/MDC/STAT/standard/MDCSTAT01501",
                "mktId": "전체",
                "trdDd": "20210621",
                "MIME Type": "application/x-www-form-urlencoded; charset=UTF-8",
                "csvxls_isNo": "false",
            }

    Returns
    -------
    converted post_prams
    Examples :
        valid_post_params : {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT01501",
            "mktId": "ALL",
            "trdDd": "20210621",
            "MIME Type": "application/x-www-form-urlencoded; charset=UTF-8",
            "csvxls_isNo": "false",
        }
    """
    converting_map = _parse_converting_map(jsp_soup)
    converting_map = _remove_empty_dict(converting_map)

    for key in post_params.keys():
        converting_by_key = converting_map.get(key, None)
        if converting_by_key is not None:
            user_input = post_params[key]
            post_params[key] = converting_by_key.get(user_input, None)
    return post_params


def get_krx_data(post_params):
    headers = {
        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/"
                      "605.1.15 (KHTML, like Gecko) Version/14.0.2 Safari/605.1.15"
    }
    url = "http://data.krx.co.kr/comm/bldAttendant/getJsonData.cmd"
    r = webio.post(url, data=post_params, headers=headers)
    try:
        return r.json()
    except json.JSONDecodeError as e:
        print(
            f"\tdata:\t{post_params}\n"
            f"error:\t{e}\n"
            f"status code:\t{r.status_code}"
            f"response:\t{r}"
        )


def _parse_converting_map(jsp_soup):
    """
    converting map is utilized to convert post_parmas into vaild_post_params to get krx data

    Parameters
    ----------
    jsp_soup : bs4.BeautifulSoup

    Returns
    -------
    converting_map : dict
        {
        "mktId": {"전체": "ALL", "KOSPI": "STK", "KOSDAQ": "KSQ", "KONEX": "KNX"},
        "trdDd": {},
        "share": {"주": "1", "천주": "2", "백만주": "3"},
        "money": {"원": "1", "천원": "2", "백만원": "3", "십억원": "4"},
        }
    """
    # 총 3개의 tag(select, label, input)에서 필요한 정보를 추출한다.
    select = jsp_soup.find_all("select")
    label = jsp_soup.find_all("label")
    input_ = jsp_soup.find_all("input")

    label_map = {}
    for i in label:
        label_map[i.attrs["for"]] = i.text

    converting_map = {}
    for i in input_:
        if i.attrs.get("value", None) != "":
            inner = converting_map.setdefault(i.attrs["name"], {})
            text = label_map.get(i.attrs.get("id", None), None)
            if text is not None:
                inner[text] = i.attrs.get("value", None)

    for select_tag in select:
        if select_tag.attrs.get("name", None) in [
            "bndClssCd",
            "isurCd",
            "idxIndCd",
            "prodId",
            "isuCd",
            "selecbox",
            "invstTpCd",
        ]:  # [14021], [15001], [15007]
            efrb_url = _parse_efrb_url(select_tag)
            result = _get_converting_map(efrb_url)
            converting_map[select_tag.attrs["name"]] = result
        elif select_tag.find_all("option") != "":
            dic = {}
            for option in select_tag.find_all("option"):
                dic[option.text] = option.attrs["value"]
            converting_map[select_tag.attrs["name"]] = dic
        else:
            converting_map[select_tag.attrs["name"]] = {
                select_tag.text: select_tag.next.attrs["value"]
            }
    return converting_map


def _remove_empty_dict(converting_map):
    """
    removes some co

    Parameters
    ----------
    converting_map : dict
        {
        "mktId": {"전체": "ALL", "KOSPI": "STK", "KOSDAQ": "KSQ", "KONEX": "KNX"},
        "trdDd": {},
        "share": {"주": "1", "천주": "2", "백만주": "3"},
        "money": {"원": "1", "천원": "2", "백만원": "3", "십억원": "4"},
        }

    Returns
    -------
        {
        'mktId': {'전체': 'ALL', 'KOSPI': 'STK', 'KOSDAQ': 'KSQ', 'KONEX': 'KNX'},
        'share': {'주': '1', '천주': '2', '백만주': '3'},
        'money': {'원': '1', '천원': '2', '백만원': '3', '십억원': '4'}
        }
    """
    no_use = []
    for key in converting_map.keys():
        if len(converting_map[key]) == 0:
            no_use.append(key)
    for no in no_use:
        converting_map.pop(no)
    return converting_map


def _get_converting_map(efrb_url):
    """
    Getting bundle from ExecuteForResourceBundle.cmd in order to make korean map.
    if user inputs '코스피200 선물' as a parameter, It needs to be converted into 'KRDRVFUK2I'.

    Parameters
    ----------
    efrb_url : str
        Execute for resource bundle url
        Examples : http://data.krx.co.kr/comm/bldAttendant/executeForResourceBundle.cmd?baseName=krx.mdc.i18n.component&key=B107.bld&type=kospi

    Returns
    -------
    Examples :
        {
         '코스피200 선물': 'KRDRVFUK2I',
         '미니코스피200 선물': 'KRDRVFUMKI',
         '코스피200 옵션': 'KRDRVOPK2I',
         '코스피200 위클리 옵션': 'KRDRVOPWKI',
         '미니코스피200 옵션': 'KRDRVOPMKI',
         '코스닥150 선물': 'KRDRVFUKQI',
         '코스닥150 옵션': 'KRDRVOPKQI',
         'KRX300 선물': 'KRDRVFUXI3',
         '변동성지수 선물': 'KRDRVFUVKI',
         '섹터지수 선물': 'KRDRVFUXAT',
         '3년국채 선물': 'KRDRVFUBM3',
         '5년국채 선물': 'KRDRVFUBM5',
         '10년국채 선물': 'KRDRVFUBMA',
         '미국달러 선물': 'KRDRVFUUSD',
         '달러플렉스 선물': 'KRDRVFXUSD',
         '미국달러 옵션': 'KRDRVOPUSD',
         '엔 선물': 'KRDRVFUJPY',
         '유로 선물': 'KRDRVFUEUR',
         '위안 선물': 'KRDRVFUCNH',
         '금 선물': 'KRDRVFUKGD',
         '주식 선물': 'KRDRVFUEQU',
         '주식 옵션': 'KRDRVOPEQU',
         '유로스톡스50 선물': 'KRDRVFUEST'
         }
    """
    soup = webio.get(efrb_url)
    the = json.loads(str(soup))
    new = the["result"]["output"]
    converting_map = {}
    for key in new:
        converting_map[key["name"]] = key["value"]
    return converting_map


def _parse_efrb_url(select_tag):
    """
    returns efrb_url to get converting map

    Parameters
    ----------
    select_tag : bs4.element.Tag
        <select id="prodId" name="prodId"></select>
        The tag is utilzed to complete efrb_url queries.
        queries : str
              "'krx.mdc.i18n.component',
                    key: 'B107.bld'"
    Returns
    -------
    efrb_url : http://data.krx.co.kr/comm/bldAttendant/executeForResourceBundle.cmd?baseName=krx.mdc.i18n.component&key=B107.bld&type=kospi
    """
    queries = str(select_tag.next.next).split("baseName:")[1].split("}")[0]
    for e in queries.split("'"):
        if "krx" in e:
            baseName = e
        elif "bld" in e:
            key = e
    efrb_url = f"http://data.krx.co.kr/comm/bldAttendant/executeForResourceBundle.cmd?baseName={baseName}&key={key}&type=kospi"
    return efrb_url



import configparser
import os

import requests


def load_api_key() -> str:
    """Binance API 키를 key.cfg에서 불러온다."""
    path = os.path.dirname(os.path.realpath(__file__))
    config = configparser.ConfigParser()
    config.read_file(open(path + "/key.cfg"))
    API_KEY = config.get("BINANCE", "BINANCE_API_KEY")
    return API_KEY


def get_data_from_api(api: str, payload: dict = None) -> dict:
    """Get 방식으로 Binance API에 요청"""
    url = "https://api.binance.com"
    headers = {
        "Content-Type": "application/json",
        "User-Agent": "Mozilla/5.0",
        "referer": "http://m.naver.com",
    }
    response = requests.get(f"{url}{api}", headers=headers, params=payload)
    return response.json()


def api_check_server_time() -> dict:
    """Binance API 요청(Check Server Time)"""
    api = "/api/v3/time"
    response = get_data_from_api(api)
    return response


def api_exchange_info() -> dict:
    """Binance API 요청(Exchange Information)"""
    api = "/api/v3/exchangeInfo"
    response = get_data_from_api(api)
    return response


def api_depth(symbol, limit=None) -> dict:
    """Binance API 요청(Order Book)"""
    api = "/api/v3/depth"
    payload = {
        "symbol": symbol,
        "limit": limit,
    }
    response = get_data_from_api(api, payload)
    return response


def api_avg_price(symbol) -> dict:
    """Binance API 요청(Current Average Price)"""
    api = "/api/v3/avgPrice"
    payload = {
        "symbol": symbol,
    }
    response = get_data_from_api(api, payload)
    return response


def api_24hr(symbol=None) -> dict:
    """Binance API 요청(24hr symbol Price Change Statistics)"""
    api = "/api/v3/ticker/24hr"
    payload = {
        "symbol": symbol,
    }
    response = get_data_from_api(api, payload)
    return response


def api_klines(symbol, interval, startTime, endTime, limit) -> dict:
    """Binance API 요청(Kline/Candlestick Data)
    Klines are uniquely identified by their open time.
    Return Examples
    1499040000000,      // Open time  -> GMT 기준 long type timestamp
    "0.01634790",       // Open
    "0.80000000",       // High
    "0.01575800",       // Low
    "0.01577100",       // Close
    "148976.11427815",  // Volume
    1499644799999,      // Close time
    "2434.19055334",    // Quote asset volume
    308,                // Number of trades
    "1756.87402397",    // Taker buy base asset volume
    "28.46694368",      // Taker buy quote asset volume
    "17928899.62484339" // Ignore.
    """
    api = "/api/v3/klines"
    payload = {
        "symbol": symbol,
        "interval": interval,
        "limit": limit,
        "startTime": startTime,
        "endTime": endTime,
    }
    response = get_data_from_api(api, payload)
    return response



from coredotfinance.binance.utils import get_date_list
from freezegun import freeze_time


@freeze_time("20210420")
def test_get_date_list():
    assert get_date_list("20210103") == [
        "20210420",
        "20210401",
        "20210301",
        "20210201",
        "20210103",
    ]



from datetime import date, datetime, timedelta

import pytz

UTC = pytz.utc


def convert_date2timestamp_sec(date: str) -> str:
    """Date(YYYYMMDD) -> UTC Timestamp(in seconds)"""
    year, month, day = int(date[0:4]), int(date[4:6]), int(date[6:8])
    timestamp = datetime(year, month, day, tzinfo=UTC).timestamp()
    return int(timestamp)


def convert_timestamp2datetime_list(timestamp_list: list()) -> list():
    datetime_list = [datetime.fromtimestamp(timestamp) for timestamp in timestamp_list]
    return datetime_list


def get_date_today() -> str:
    """Get date of today(YYYYMMDD)"""
    today = date.today()
    return today.strftime("%Y%m%d")


def get_date_past_days_ago(days: int = 365) -> str:
    """Get date of past days ago"""
    today = date.today()
    past_days_ago = today - timedelta(days)
    return past_days_ago.strftime("%Y%m%d")

def set_index_datetime(df):
    if "datetime" in df:
        df = df.set_index("datetime").sort_index(ascending=False)
    elif "일시" in df:
        df = df.set_index("일시").sort_index(ascending=False)
    return df


# -*- coding: utf-8 -*-
from coredotfinance.krx.core.krx_website.info import Info


class Stock(Info):

    def __init__(
        self, code, start, end, date, symbol, **kwargs
    ):
        """
        주가
        :param code:
        :param start:
        :param end, 'index':
        :param day:
        :param division:
        :param item:
        """
        self.code = code
        self.start = start
        self.end = end
        self.date = date
        self.symbol = symbol
        self.detail = kwargs.get("detail", None)
        self.trade_index = kwargs.get("trade_index", None)
        self.trade_check = kwargs.get("trade_check", None)
        self.search_type = kwargs.get("search_type", None)
        self.investor = kwargs.get("investor", None)
        self.inquiry = kwargs.get("inquiry", None)
        self.search_type = kwargs.get("search_type", "전종목")
        self.sort_type = kwargs.get("sort_type", "종목명")
        self.isuLmtRto = kwargs.get("no_foreign_only", None)
        self.business = kwargs.get("business", None)
        self.company = kwargs.get("company", None)

        addition_item = kwargs.get("addition_item", None)
        if addition_item:
            addition_item = [item.upper() for item in addition_item]
            self.etf = "ETF" if "ETF" in addition_item else None
            self.etn = "ETN" if "ETN" in addition_item else None
            self.elw = "ELW" if "ELW" in addition_item else None

        if symbol:
            self.data_nm, self.data_cd, self.data_tp = self.autocomplete(symbol, 'stock')
        division = kwargs.get('division', None)
        self.division = "전체" if division is None else division.upper()

        self.code_to_function = {
            "12001": self.price_of_entire_item,
            "12002": self.fluc_of_entire_item,
            "12003": self.trend_of_item_price,
            "12004": self.trend_of_item_price_by_month,
            "12005": self.info_of_entire_itme,
            "12006": self.option_list_of_entire,
            "12007": "Not Now",
            "12008": self.trade_perform_by_invastor,
            "12009": self.trade_perform_by_item,
            "12010": self.top_item_per_investor,
            "12011": self.block_trading_last_day,
            "12012": self.program_traing,
            "12013": self.price_of_REITs,
            "12014": self.price_of_mutual_fund,
            "12015": self.price_of_ship_investor,
            "12016": self.price_of_infra_investor,
            "12017": self.price_of_certificate,
            "12018": self.price_of_warranty,
            "12019": self.price_of_subscription_warranty,
            "12020": self.detail_search_of_listed_company,
            "12021": self.per_pbr_dividend_of_stock,
            "12022": self.holding_amount_of_foreigner,
            "12023": self.holding_amount_of_foreigner_by_item,
            "12024": self.distribution_per_business,
            "12025": self.stock_and_business_table,
            "12026": self.substitution_price_of_stock,
            "12027": self.substitution_price_of_beneficiary_certificate,
            "12028": self.substitution_price_of_mutual_fund,
        }
        self.get_requested_data = self.code_to_function[code]

    def price_of_entire_item(self):
        """전종목 시세 [12001]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT01501",
            "mktId": self.division,
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def fluc_of_entire_item(self):
        """전종목 등락률 [12002]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT01602",
            "mktId": self.division,
            "strtDd": self.start,
            "endDd": self.end,
            "adjStkPrc_check": self.detail,
        }
        return self.update_requested_data(data)

    def trend_of_item_price(self):
        """개별종목 시세 추이 [12003]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT01701",
            "tboxisuCd_finder_stkisu0_2": f"{self.data_tp}/{self.data_nm}",
            "isuCd": self.data_cd,
            "isuCd2": self.data_tp,
            "codeNmisuCd_finder_stkisu0_2": self.data_nm,
            "param1isuCd_finder_stkisu0_2": "STK",
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def trend_of_item_price_by_month(self):
        """개별종목 시세 추이 [12004]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT01801",
            "tboxisuCd_finder_stkisu0_3": f"{self.data_tp}/{self.data_nm}",
            "isuCd": self.data_cd,
            "isuCd2": self.data_cd,
            "codeNmisuCd_finder_stkisu0_3": self.data_nm,
            "param1isuCd_finder_stkisu0_3": "STK",
            "strtYy": self.start[:4],
            "strtMm": self.start[4:6],
            "endYy": self.end[:4],
            "endMm": self.end[:4:6],
            "strtYymm": self.start[:6],
            "endYymm": self.end[:6],
        }
        return self.update_requested_data(data)

    def info_of_entire_itme(self):
        """전체 종목 기본 정보 [12005]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT01901",
            "mktId": self.division,
        }
        return self.update_requested_data(data)

    def option_list_of_entire(self):
        """전종목 지정 내역 [12006]"""
        data = {"bld": "dbms/MDC/STAT/standard/MDCSTAT02001", "mktId": self.division}
        return self.update_requested_data(data)

    def total_info_of_stock(self):
        """개별종목 종합정보 [12007]"""
        pass

    def trade_perform_by_invastor(self):
        """투자자별 거래실적 [12008]"""
        # 기간합계(1)/일별추이(2)(inqTqCd) - 거래량(1)/거래대금(2)(trdValVol), 매도(1)/매수(2)/순매수(3)(askBid)
        # 기간합계는 detailView, trdValVol, askBid 가 없음.
        # KOSDAQ, KONEX는 ETF, ETN, ELW가 없음
        if self.detail == "상세보기":
            bld = "dbms/MDC/STAT/standard/MDCSTAT02203"
        elif self.search_type == "일별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT02202"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT02201"
        data = {
            "bld": bld,
            "inqTpCd": self.search_type,
            "trdVolVal": self.trade_index,
            "askBid": self.trade_check,
            "mktId": self.division,
            "etf": self.etf,
            "etn": self.etn,
            "elw": self.elw,
            "strtDd": self.start,
            "endDd": self.end,
            "detailView": self.detail,
        }
        return self.update_requested_data(data)

    def trade_perform_by_item(self):
        """투자자별 거래실적(개별종목) [12009]"""
        if self.detail == "상세보기":
            bld = "dbms/MDC/STAT/standard/MDCSTAT02303"
        elif self.search_type == "일별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT02302"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT02301"
        data = {
            "bld": bld,
            "inqTpCd": self.search_type,
            "trdVolVal": self.trade_index,
            "askBid": self.trade_check,
            "tboxisuCd_finder_stkisu0_3": f"{self.data_tp}/{self.data_nm}",
            "isuCd": self.data_cd,
            "isuCd2": self.data_cd,
            "codeNmisuCd_finder_stkisu0_3": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
            "detailView": self.detail,
        }
        return self.update_requested_data(data)

    def top_item_per_investor(self):
        """투자자별 순매수상위종목 [12010]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT02401",
            "mktId": self.division,
            "invstTpCd": self.investor,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def block_trading_last_day(self):
        """대량매매(전일) [12011]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT02501",
            "mktId": self.division,
        }
        return self.update_requested_data(data)

    def program_traing(self):
        """프로그램 매매 [12012]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT02601",
            "mktId": self.division,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def price_of_REITs(self):
        """REITs시세 [12013]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT02701",
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def price_of_mutual_fund(self):
        """뮤추얼펀드 시세 [12014]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT02801",
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def price_of_ship_investor(self):
        """선박투자회사 시세 [12015]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT02901",
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def price_of_infra_investor(self):
        """인프라투융자회사 시세 [12016]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT03001",
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def price_of_certificate(self):
        """수익증권 시세 [12017]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT03101",
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def price_of_warranty(self):
        """신주인수권증권 시세 [12018]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT03201",
            "mktId": self.division,
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def price_of_subscription_warranty(self):
        """신주인수권증서 시세 [12019]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT03301",
            "mktId": self.division,
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def detail_search_of_listed_company(self):
        """상장회사 상세검색 [12020]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT03402",
            "mktTpCd": self.division,
            "tboxisuSrtCd_finder_listisu0_3": f"{self.data_tp}/{self.data_nm}",
            "isuSrtCd": self.data_cd,
            "isuSrtCd2": "A" + self.data_tp,
            "codeNmisuSrtCd_finder_listisu0_3": self.data_nm,
            "sortType": self.sort_type,
        }
        return self.update_requested_data(data)

    def per_pbr_dividend_of_stock(self):
        """ "PER/PBR/배당수익률(개별종목) [12021]"""
        if self.search_type == "개별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT03502"
        else:
            self.data_nm, self.data_cd, self.data_tp = None, None, None
            bld = "dbms/MDC/STAT/standard/MDCSTAT03501"

        data = {
            "bld": bld,
            "searchType": self.search_type,
            "mktId": self.division,
            "trdDd": self.date,
            "tboxisuCd_finder_stkisu0_0": f"{self.data_tp}/{self.data_nm}",
            "isuCd": self.data_cd,
            "isuCd2": self.data_cd,
            "codeNmisuCd_finder_stkisu0_0": self.data_nm,
            "param1isuCd_finder_stkisu0_0": self.division,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def holding_amount_of_foreigner(self):
        """외국인보유량 추이 [12022]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT03601",
            "mktId": self.division,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def holding_amount_of_foreigner_by_item(self):
        """외국인보유량(개별종목) [12023]"""
        if self.search_type == "개별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT03702"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT03701"

        data = {
            "bld": bld,
            "searchType": self.search_type,
            "mktId": self.division,
            "trdDd": self.date,
            "tboxisuCd_finder_stkisu0_0": f"{self.data_tp}/{self.data_nm}",
            "isuCd": self.data_cd,
            "isuCd2": self.data_cd,
            "codeNmisuCd_finder_stkisu0_0": self.data_nm,
            "param1isuCd_finder_stkisu0_0": self.division,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def distribution_per_business(self):
        """
        업종별 분포 [12024]
        :arg
            [searchType = 1 (전종목)]
                search_type, day, division
            [searchType = 2 (개별추이)]
                search_type, division, business, start, end
        division in (KOSPI, KOSDAQ)
        """
        if self.search_type == "개별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT03802"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT03801"

        data = {
            "bld": bld,
            "searchType": self.search_type,
            "mktId": self.division,
            "trdDd": self.date,
            "idxIndCd": self.inquiry,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def stock_and_business_table(self):
        """
        업종분류 현황 [12025]
        :arg
            division, day
        division in (KOSPI, KOSDAQ)
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT03901",
            "mktId": self.division,
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def substitution_price_of_stock(self):
        """
        주식 대용가 [12026]
        :arg
            [searchType = 1 (전종목)]
                search_type, day, division
            [searchType = 2 (개별추이)]
                search_type, dividion, business, start, end
        """
        if self.search_type == "개별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT04002"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT04001"
        data = {
            "bld": bld,
            "searchType": self.search_type,
            "mktId": self.division,
            "trdDd": self.date,
            "tboxisuCd_finder_stkisu0_1": f"{self.data_tp}/{self.data_nm}",
            "isuCd": self.data_cd,
            "isuCd2": self.data_tp,
            "codeNmisuCd_finder_stkisu0_1": self.data_nm,
            "param1isuCd_finder_stkisu0_1": "STK",
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def substitution_price_of_beneficiary_certificate(self):
        """
        수익증권 대용가 [12027]
        :arg
            [searchType = 1 (전종목)]
                search_type, day, division
            [searchType = 2 (개별추이)]
                search_type, start, end, company, certificate
        """
        if self.search_type == "전종목":
            strtYy = self.date[:4]
            strtMm = self.date[4:6]
            bld = "dbms/MDC/STAT/standard/MDCSTAT04101"
        else:
            strtYy = self.start[:4]
            strtMm = self.start[4:6]
            bld = "dbms/MDC/STAT/standard/MDCSTAT04102"
        endYy = self.end[:4]
        endMm = self.end[4:6]

        data = {
            "bld": bld,
            "strtYy": strtYy,
            "strtMm": strtMm,
            "searchType": self.search_type,
            "comNm": self.company,
            "isuNm": self.item,
            "endYy": endYy,
            "endMm": endMm,
        }

        return self.update_requested_data(data)

    def substitution_price_of_mutual_fund(self):
        """
        뮤추얼펀드 대용가 [12028]
        :arg
            [searchType = 1 (전종목)]
                search_type, day, division
            [searchType = 2 (개별추이)]
                search_type, start, end, company, certificate
        """
        if self.search_type == "전종목":
            strtYy = self.date[:4]
            strtMm = self.date[4:6]
            bld = "dbms/MDC/STAT/standard/MDCSTAT04201"
        else:
            strtYy = self.start[:4]
            strtMm = self.start[4:6]
            bld = "dbms/MDC/STAT/standard/MDCSTAT04202"
        endYy = self.end[:4]
        endMm = self.end[4:6]

        data = {
            "bld": bld,
            "strtYy": strtYy,
            "strtMm": strtMm,
            "searchType": self.search_type,
            "comNm": self.company,
            "isuNm": self.item,
            "endYy": endYy,
            "endMm": endMm,
        }

        return self.update_requested_data(data)



import datetime
import os
import time

import numpy as np
import pandas as pd

from coredotfinance.util import datetime_util
from coredotfinance.util import dataframe_util

from coredotfinance.binance.api import (
    api_24hr,
    api_avg_price,
    api_depth,
    api_exchange_info,
    api_klines,
)
from coredotfinance.binance.utils import get_date_list


def get_symbols() -> list:
    """Binance의 Symbol List 리턴"""
    response = api_exchange_info()
    symbol_list = [
        response["symbols"][i]["symbol"] for i in range(len(response["symbols"]))
    ]
    return symbol_list


def get_current_price(symbol) -> float:
    """대상 Symbol의 현재 가격 리턴"""
    print(symbol.upper())
    response = api_avg_price(symbol.upper())
    return float(response.get("price"))


def get_orderbook(symbol, limit=None) -> pd.DataFrame:
    """대상 Symbol의 호가창(DataFrame) 리턴"""
    print(symbol.upper())
    response = api_depth(symbol.upper(), limit=limit)
    bids = np.array(response["bids"])
    asks = np.array(response["asks"])
    concat = np.concatenate((bids, asks), axis=1)
    df = pd.DataFrame(
        concat, columns=["bid_price", "bid_volume", "ask_price", "ask_volume"]
    )
    df = dataframe_util.rename_cols2kor(df)
    return df


def get_24hr_all_price() -> pd.DataFrame:
    """모든 Symbol의 24시간 동안의 가격 정보(DataFrame) 리턴 (거래대금순 내림차순 정렬)"""
    response = api_24hr()
    df = pd.DataFrame(response)
    df["tradingValue"] = df["volume"].astype(float) * df["weightedAvgPrice"].astype(
        float
    )
    isUSDT = df["symbol"].str.contains(".USDT", regex=True)
    cols = [
        "symbol",
        "priceChange",
        "priceChangePercent",
        "openPrice",
        "highPrice",
        "lowPrice",
        "lastPrice",
        "volume",
        "tradingValue",
    ]
    df = (
        df.loc[isUSDT, cols]
        .sort_values(by=["tradingValue"], ascending=False)
        .reset_index(drop=True)
    )
    df = dataframe_util.rename_cols2kor(df)
    return df


def get_ohlcv(
    symbol: str = "BTCUSDT", interval="1d", start=None, end=None, limit=1000
) -> pd.DataFrame:
    """대상 symbol의 가격 정보(DataFrame) 리턴

    Parameters
    ----------
    symbol : str, optional
        Binance Symbol, by default "BTCUSDT"
    interval : str, optional
        조회 간격 설정, by default "1d"
        (1m, 3m, 5m, 15m, 30m, 1h, 2h, 4h, 6h, 8h, 12h, 1d, 3d, 1w, 1M)
    start : str, optional
        조회 시작 날짜(YYYYMMDD), by default 최근 날짜
    end : str, optional
        조회 끝 날짜(YYYYMMDD), by default 최근 날짜
    limit : int, optional
        조회 개수, by default 1000

    Returns
    -------
    pd.DataFrame
        대상 symbol의 조회 조건에 맞는 일시별 시가/고가/저가/종가/거래량 DataFrame
    """
    if start:
        start = datetime_util.convert_date2timestamp_sec(start) * 1000  # s -> ms
    if end:
        end = datetime_util.convert_date2timestamp_sec(end) * 1000  # s -> ms

    ohlcv = api_klines(symbol.upper(), interval, start, end, limit)

    df = pd.DataFrame(ohlcv).iloc[:, :6]
    df.columns = ["datetime", "open", "high", "low", "close", "volume"]
    df["datetime"] = pd.to_datetime(df["datetime"], unit="ms")

    df = dataframe_util.rename_cols2kor(df)
    df = datetime_util.set_index_datetime(df)

    if "d" in interval or "h" in interval:
        df.tz_localize("UTC")

    print(symbol.upper())
    return df


def get_hourly_ohlcv_to_pickle(symbol_list, start_day, dir):
    """Symbol List에 대해 지정된 시작날짜부터의 1시간 간격 가격정보(OHLCV)를 지정된 폴더에 pickle 파일로 저장"""
    date_list = get_date_list(start_day)
    for symbol in symbol_list:
        outdir = f"{dir}/binance/pickles_{symbol}"  # Symbol 별로 폴더 분류
        if not os.path.exists(outdir):  # 폴더가 존재하지 않을경우 폴더 생성
            os.makedirs(outdir)
        for idx, date in enumerate(date_list):
            if not idx == 0:  # idx==0이면, idx-1==-1이 되므로 제외
                start, end = date_list[idx], date_list[idx - 1]
                df = get_ohlcv(symbol, interval="1h", start=start, end=end, limit=1000)
                if df.shape[0] == 0:  # DataFrame에 Data가 없는 경우 For Loop 종료
                    print(f"NoData : {symbol.upper()}_{date[:-2]}")
                    break
                df.to_pickle(f"{outdir}/{symbol}_{date[:-2]}.pickle")
                time.sleep(1)  # API에서 IP Ban 방지하기 위하여 1초 Delay


def get_recent_ohlcv_to_pickle(symbol_list, dir):
    """Symbol List에 대해 전일 기준 해당월의 1시간 간격 가격정보(OHLCV)를 지정된 폴더에 pickle 파일로 저장"""
    yesterday = datetime.datetime.now() - datetime.timedelta(days=1)
    first_day = datetime.datetime(yesterday.year, yesterday.month, 1).strftime("%Y%m%d")
    for symbol in symbol_list:
        outdir = f"{dir}/binance/pickles_{symbol}"  # Symbol 별로 폴더 분류
        if not os.path.exists(outdir):  # 폴더가 존재하지 않을경우 폴더 생성
            os.makedirs(outdir)
        df = get_ohlcv(symbol, interval="1h", start=first_day)
        df.to_pickle(f"{outdir}/{symbol}_{first_day[:-2]}.pickle")
        time.sleep(1)  # API에서 IP Ban 방지하기 위하여 1초 Delay



import pickle

# test_is to avoid a conflict
# btw same file name(.py) and function name(def .

"""
bundle_for_test 는 coredotcoredotfinance.krx.'15001', item='코스피200 선물', market='전체') 20210430 에 만들었다.
"""

with open("test/test_data_reader.bin", "rb") as f:
    data = pickle.load(f)

requested_data = data["requested_data"]
mdcstat = data["mdcstat"]
jsp_soup = data["jsp_soup"]
krx_data = data["krx_data"]
efrb_url = data["efrb_url"]
resource_bundle = data["resource_bundle"]
converting_map = data["converting_map"]
len_zero_converting_map = data["len_zero_converting_map"]
valid_requested_data = data["valid_requested_data"]
jsGrid_dict = data["jsGrid_dict"]

# FIXME: tag can not be pickled.
# select_tag = bs(data['select_tag'])
# div_tag = bs(data['div_tag'])
# div_map = data['div_map']


def test_parse_mdcstat():
    assert parse_mdcstat(requested_data) == mdcstat


def test_get_jsp_soup():
    # Removes date data in jsp_soup
    result = str(get_jsp_soup(mdcstat))
    result = re.sub(r"SEARCH_COMPONENT__[0-9]*", "", result)
    result = re.sub(f'value="[0-9]*"', "", result)
    answer = str(jsp_soup)
    answer = re.sub(r"SEARCH_COMPONENT__[0-9]*", "", answer)
    answer = re.sub(f'value="[0-9]*"', "", answer)
    assert result == answer


def test_convert_valid_requested_data():
    assert (
        convert_valid_requested_data(jsp_soup, requested_data) == valid_requested_data
    )


def test_parse_converting_map():
    assert parse_converting_map(jsp_soup) == converting_map


def test_remove_len_zero():
    assert remove_len_zero(converting_map) == len_zero_converting_map


def test_get_resource_bundle():
    assert get_resource_bundle(efrb_url) == resource_bundle


def test_parse_jsGride_dict():
    assert parse_jsGrid_dict(jsp_soup) == jsGrid_dict


"""
def test_parse_efrb_url():
    assert parse_efrb_url(select_tag) == efrb_url
"""

# FIXME : No test data! If someone finds some functions using table_tag, Add that or tell Gukmoon.
"""
def test_parse_table_map():
    assert parse_table_map(table_tag) == table_map
"""

"""
def test_parse_div_map():
    assert parse_div_map(div_tag) == div_map
"""



# -*- coding: utf-8 -*-
from coredotfinance.krx.core.krx_website.info import Info


class Derivative(Info):
    def __init__(self, code, start, end, day, item, code_to_function, kwargs):
        super().__init__(start, end, day)
        self.get_requested_data = code_to_function[code]
        if code in ["15002"]:
            self.data_nm, self.data_cd, self.data_tp = self.autocomplete(
                item, "derivative"
            )
        self.item = item
        self.inquiry = kwargs.get("inquiry", None)
        self.trade_index = kwargs.get("trade_index", None)
        self.trade_check = kwargs.get("trade_check", None)
        self.market = kwargs.get("market", None)
        self.right_type = kwargs.get("right_type", None)
        self.detail = kwargs.get("detail", None)
        self.search_type = kwargs.get("search_type", None)


class ItemPrice(Derivative):
    def __init__(self, code, start, end, day, item, **kwargs):
        code_to_function = {
            "15001": self.price_of_entire_item,
            "15002": self.price_trend_of_item,
            "15003": self.price_trend_of_futures,
        }
        super().__init__(code, start, end, day, item, code_to_function, kwargs)

    def price_of_entire_item(self):
        """전종목 시세[15001]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT12501",
            "trdDd": self.day,
            "prodId": self.item,
            "mktTpCd": self.market,
        }
        return self.update_requested_data(data)

    def price_trend_of_item(self):
        """개별시세 추이[15002]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT12601",
            "strtDd": self.start,
            "endDd": self.end,
            "tboxisuCd_finder_drvprodisu0_1": f"{self.data_tp}/ {self.data_nm}",
            "isuCd": self.data_cd,
        }
        return self.update_requested_data(data)

    def price_trend_of_futures(self):
        """최근월물 시세 추이(선물)[15003]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT12701",
            "prodId": self.item,
            #'subProdId': self.detail, # subProdId를 불러오는데 문제가 있었
            "strtDd": self.start,
            "endDd": self.end,
            "mktTpCd": self.market,
        }
        return self.update_requested_data(data)


class ItemInfo(Derivative):
    def __init__(self, code, start, end, day, item, **kwargs):
        code_to_function = {
            "15004": self.info_of_entire_item,
            "15005": self.info_of_item,
        }
        super().__init__(code, start, end, day, item, code_to_function, kwargs)

    def info_of_entire_item(self):
        """전종목 기본정보[15004]"""
        data = {"bld": "dbms/MDC/STAT/standard/MDCSTAT12801", "prodId": self.item}
        return self.update_requested_data(data)

    def info_of_item(self):
        """개별종목 종합정보[15005]"""
        return "Not now"


class TradePerform(Derivative):
    def __init__(self, code, start, end, day, item, **kwargs):
        code_to_function = {
            "15006": self.trade_perform_of_entire_item,
            "15007": self.trade_perform_ber_invastor,
            "15008": self.trend_of_consulted_big_trade_performance,
            "15009": self.trade_performance_of_basic_asset,
        }
        super().__init__(code, start, end, day, item, code_to_function, kwargs)

    def trade_perform_of_entire_item(self):
        """전체상품 거래실적[15006]"""
        data = {"bld": "dbms/MDC/STAT/standard/MDCSTAT13001", "trdDd": self.day}
        return self.update_requested_data(data)

    def trade_perform_ber_invastor(self):
        """투자자별 거래실적[15007]"""
        if self.search_type == "개별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT13102"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT13101"
        data = {
            "bld": bld,
            "isuCd": self.item,
            "strtDd": self.start,
            "endDd": self.end,
            "inqTpCd": self.search_type,
            "prtType": self.trade_index,
            "prtCheck": self.trade_check,
            "juya": self.market,
        }
        return self.update_requested_data(data)

    def trend_of_consulted_big_trade_performance(self):
        """협의대향거래실적 추이[15008]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT13201",
            "isuCd": self.item,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def trade_performance_of_basic_asset(self):
        """기초자산별 거래실적(주식/선물/옵션)[15009]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT13301",
            "secugrpId": self.item,
            "isuOpt": self.right_type,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)


class Detail(Derivative):
    def __init__(self, code, start, end, day, item, **kwargs):
        code_to_function = {
            "15010": self.future_trend_of_basis,
            "15011": self.trend_of_implied_volatility,
            "15012": self.trend_of_pc_ratio,
            "15013": self.price_table_per_expiration_and_discount,
            "15014": self.trend_of_pork_price,
            "15015": self.sudden_change_of_setting_price,
            "15016": self.ratio_of_low_exercise_of_right,
        }
        super().__init__(code, start, end, day, item, code_to_function, kwargs)

    def future_trend_of_basis(self):
        """베이시스 추이(선물)[15010]"""
        if self.search_type == "개별종목":
            bld = "dbms/MDC/STAT/standard/MDCSTAT13402"
            data_cd, data_nm, data_tp = self.autocomplete(self.item, "derivative")
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT13401"
            data_cd, data_nm, data_tp = None, None, None
        data = {
            "bld": bld,
            "secugrpId": self.search_type,
            "prodId": self.item,
            "expmmNo": self.detail,
            "tboxisuCd_finder_drvfuprodisu1_1": f"{data_tp}/{data_nm}",
            "isuCd": data_cd,
            "isuCd2": self.item,
            "codeNmisuCd_finder_drvfuprodisu1_1": data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def trend_of_implied_volatility(self):
        """내재변동성 추이(옵션)[15011]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT13501",
            "prodId": self.item,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def trend_of_pc_ratio(self):
        """P/C Ratio 추이(옵션)[15012]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT13601",
            "prodId": self.item,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def price_table_per_expiration_and_discount(self):
        """행사가격/만기별 가격표(옵션)[15013]"""
        data = {"bld": "dbms/MDC/STAT/standard/MDCSTAT13701", "prodId": self.item}
        return "Not now"  # 표를 띄우는 것이 쉽지 않음..

    def trend_of_pork_price(self):
        """돈육시세 동향[15014]"""
        if self.inquiry == "현재가":
            bld = "dbms/MDC/STAT/standard/MDCSTAT13801"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT13802"
        data = {"bld": bld, "strtDd": self.start, "endDd": self.end}
        return self.update_requested_data(data)

    def sudden_change_of_setting_price(self):
        """최종결제가격 급변[15015] (기간별조회)"""
        year = self.day[:4]
        month = self.day[4:6]
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT14001",
            "trdYy": year,
            "trdMm": month,
            "cpType": self.inquiry,
        }
        #  최종결제가격 급변은 표를 띄우는 것이 쉽지 않음..
        #  데이터가 jsp 파일 안에 있음
        return self.update_requested_data(data)

    def ratio_of_low_exercise_of_right(self):
        """낮은 권리행사 배율[15016] (기간별조회)"""
        str_year = self.start[:4]
        str_month = self.start[4:6]
        end_year = self.end[:4]
        end_month = self.end[4:6]
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT14101",
            "strtYy": str_year,
            "strtMm": str_month,
            "endYy": end_year,
            "endMm": end_month,
        }
        #  최종결제가격 급변은 표를 띄우는 것이 쉽지 않음..
        #  데이터가 jsp 파일 안에 있음
        #  데이터 단위가 %인데 표시되어 있지 않음
        return self.update_requested_data(data)



from datetime import datetime, timedelta

from coredotfinance.binance.api import api_check_server_time
from coredotfinance.binance.binance import (
    get_24hr_all_price,
    get_current_price,
    get_ohlcv,
    get_orderbook,
    get_symbols,
)
from coredotfinance.binance.utils import get_date_list
from freezegun import freeze_time


# crypto.binance.api.py
def test_api_check_server_time():
    time_api = datetime.fromtimestamp(int(api_check_server_time()["serverTime"]) / 1000)
    time_now = datetime.now()
    assert (time_api - time_now) < timedelta(seconds=1)


# crypto.binance.binance.py
def test_get_symbols():
    assert len(get_symbols()) > 0


def test_get_current_price():
    assert get_current_price("BTCUSDT") > 0


def test_get_orderbook():
    assert len(get_orderbook("BTCUSDT")) > 0


def test_24hr_all_price():
    df = get_24hr_all_price()
    assert len(df) > 0
    assert df["거래대금"].max() == df["거래대금"][0]


def test_get_ohlcv_klines():
    df = get_ohlcv("BTCUSDT")
    assert len(df) > 0
    assert df.columns.tolist() == ["시가", "고가", "저가", "종가", "거래량"]


@freeze_time("20210420")
def test_get_date_list():
    assert get_date_list("20210103") == [
        "20210420",
        "20210401",
        "20210301",
        "20210201",
        "20210103",
    ]



import requests
from bs4 import BeautifulSoup as bs


def get(url, params=None):
    """
    requests data to the server as get-type and returns bs4.BeautifulSoup

    Parameters
    ----------
    url : str
    params : dict

    Returns
    -------
    bs4.BeautifulSoup
    """
    res = requests.get(url, params=params)
    status_ok(res)
    return bs(res.content, 'html.parser')


def post(url, data, headers=None, soup=False):
    """
    requests data to the server as post-type and returns bs4.BeautifulSoup.

    Parameters
    ----------
    url : str
    headers : dict
    data : dict

    Returns
    -------
    bs4.BeautifulSoup
    """
    res = requests.post(url, headers=headers, data=data)
    status_ok(res)
    if soup:
        return bs(res.content, 'html.parser')
    else:
        return res


def soup(txt):
    return bs(txt, 'html.parser')


def status_ok(res):
    if res.status_code != 200:
        raise ConnectionError(f"response is not 200 from the server.\nstatus_code : {res.status_code}")
    else:
        pass



import re
import os

FILE_PATH = os.path.dirname(os.path.realpath(__file__))


def get_korean_columns(jsp_soup, mdcstat):
    """
    data.krx.co.kr returns data with not friendly-key name.
    Korean_columns is utilzed to convert not friendly-key name into Korean word.

    Parameters
    ----------
    jsp_soup : bs4.BeautifulSoup
    mdcstat : str

    Returns
    -------
    map_ : dict
    Examples :
    {
     'ISU_SRT_CD': '종목코드',
     'ISU_NM': '종목명',
     'TDD_CLSPRC': '종가',
     'CMPPREVDD_PRC': '대비',
     'TDD_OPNPRC': '시가',
     'TDD_HGPRC': '고가',
     'TDD_LWPRC': '저가',
     'SPOT_PRC': '현물가',
     'SETL_PRC': '정산가',
     'ACC_TRDVOL': '거래량',
     'ACC_TRDVAL': '거래대금',
     'ACC_OPNINT_QTY': '미결제약정'
    }
    """
    map_ = {}
    jsGrid_dict = _parse_jsGrid_dict(jsp_soup)
    jsGrid = jsGrid_dict[mdcstat]

    table_tag = jsp_soup.find("table", {"id": jsGrid})
    div_tag = jsp_soup.find("div", {"id": jsGrid})

    if table_tag:
        table_map = _parse_table_map(table_tag)
        map_.update(table_map)
    if div_tag:
        div_map = _parse_div_map(div_tag)
        map_.update(div_map)
    return map_


def _parse_jsGrid_dict(jsp_soup):
    """
    Parameters
    ----------
    jsp_soup : bs4.BeautifulSoup

    Returns
    -------
    Examples:
        {
        'MDCSTAT12501': ['jsGrid_MDCSTAT125_0'],
        'MDCSTAT12502': ['jsGrid_MDCSTAT125_1']
        }
    """
    jscode_tag_list = jsp_soup.find_all("script")
    for tag in jscode_tag_list:
        # TODO : change the variable, s
        #  What does s mean?
        if "jsGrid" in str(tag):
            jscode = tag
            break
    jscode_str = str(jscode)
    mdcstat_list = re.findall(
        r"template: \$content\.select\(\'\#(jsGrid_MDCSTAT[0-9]*\_[0-9])", jscode_str
    )
    bld_list = re.findall(r"bld: \'dbms/MDC/STAT/standard/(MDCSTAT[0-9]*)", jscode_str)
    jsGrid_dict = {}
    for mdcstat, bld in zip(mdcstat_list, bld_list):
        lis = jsGrid_dict.setdefault(bld, [])
        lis.append(mdcstat)
    return jsGrid_dict


def _parse_table_map(table_tag):
    table_map = {}
    for tr in table_tag.find_all("tr"):
        th = tr.find_all("th")
        td = tr.find_all("td")
        for name, id in zip(th, td):
            table_map[id.attrs["data-bind"]] = name.text
    return table_map


def _parse_div_map(div_tag):
    """
    Parameters
    ----------
    div_tag: bs4.element.Tag
        Examples : '15001'
            <div id="jsGrid_MDCSTAT125_0">
            <table>
            <thead>
            <tr>
            <th align="center" name="ISU_SRT_CD" scope="col" width="100px">종목코드</th>
            <th name="ISU_NM" scope="col" width="220px">종목명</th>
            <th align="right" name="TDD_CLSPRC" scope="col" width="90px">종가</th>
            <th align="right" name="CMPPREVDD_PRC" scope="col" width="90px">대비</th>
            <th align="right" name="TDD_OPNPRC" scope="col" width="90px">시가</th>
            <th align="right" name="TDD_HGPRC" scope="col" width="90px">고가</th>
            <th align="right" name="TDD_LWPRC" scope="col" width="90px">저가</th>
            <th align="right" name="SPOT_PRC" scope="col" width="100px">현물가</th>
            <th align="right" name="SETL_PRC" scope="col" width="100px">정산가</th>
            <th align="right" name="ACC_TRDVOL" scope="col" width="120px">거래량</th>
            <th align="right" name="ACC_TRDVAL" scope="col" width="130px">거래대금</th>
            <th align="right" name="ACC_OPNINT_QTY" scope="col" width="120px">미결제약정</th>
            </tr>
            </thead>
            <tbody>
            <tr>
            <td bind="ISU_SRT_CD" name="ISU_SRT_CD"></td>
            <td bind="ISU_NM" name="ISU_NM"></td>
            <td bind="TDD_CLSPRC" name="TDD_CLSPRC"></td>
            <td bind="CMPPREVDD_PRC" name="CMPPREVDD_PRC"></td>
            <td bind="TDD_OPNPRC" name="TDD_OPNPRC"></td>
            <td bind="TDD_HGPRC" name="TDD_HGPRC"></td>
            <td bind="TDD_LWPRC" name="TDD_LWPRC"></td>
            <td bind="SPOT_PRC" name="SPOT_PRC"></td>
            <td bind="SETL_PRC" name="SETL_PRC"></td>
            <td bind="ACC_TRDVOL" name="ACC_TRDVOL"></td>
            <td bind="ACC_TRDVAL" name="ACC_TRDVAL"></td>
            <td bind="ACC_OPNINT_QTY" name="ACC_OPNINT_QTY"></td>
            </tr>
            </tbody>
            </table>
            </div>
    Returns
    -------
    div_map : dict
    Examples :
        {
         'ISU_SRT_CD': '종목코드',
         'ISU_NM': '종목명',
         'TDD_CLSPRC': '종가',
         'CMPPREVDD_PRC': '대비',
         'TDD_OPNPRC': '시가',
         'TDD_HGPRC': '고가',
         'TDD_LWPRC': '저가',
         'SPOT_PRC': '현물가',
         'SETL_PRC': '정산가',
         'ACC_TRDVOL': '거래량',
         'ACC_TRDVAL': '거래대금',
         'ACC_OPNINT_QTY': '미결제약정'
        }
    """
    thead = div_tag.thead
    if thead is None:
        return {}
    tag_list = []
    tag_list.extend(thead.find_all("th"))
    tag_list.extend(thead.find_all("td"))

    div_map = {}
    for i in tag_list:
        div_map[i.attrs["name"]] = {
            "text": i.text,
            "parent": i.attrs.get("parent", None),
        }

    p_list = set()
    for key in div_map:
        p = div_map[key]["parent"]

        if p is not None:
            child_name = div_map[key]["text"]
            parent_name = div_map[p]["text"]
            div_map[key]["text"] = f"{parent_name}//{child_name}"

            p_list.add(p)
    [div_map.pop(p) for p in p_list]
    for d in div_map:
        div_map[d] = div_map[d]["text"]
    return div_map



import pandas as pd
from coredotfinance.krx.api.data_reader import data_reader
from coredotfinance.krx.api.tool import (
    convert_stock_name2ticker,
    convert_stock_ticker2name,
    get_adjusted_price,
    get_df_12009,
    get_stock_info,
    get_stock_pack,
)


def test_convert_stock_name2ticker():
    assert convert_stock_name2ticker("삼성전자") == "005930"


def test_convert_stock_ticker2name():
    assert convert_stock_ticker2name("005930") == "삼성전자"


def test_get_stock_info():
    get_stock_info().시장구분.unique == ["KOSPI", "KOSDAQ"]


def test_get_stock_pack():
    col_12003 = set(data_reader("12003", item="삼성전자").columns)
    col_12021 = set(data_reader("12021", item="삼성전자", search_type="개별추이").columns)
    col_12023 = set(data_reader("12023", item="삼성전자", search_type="개별추이").columns)
    col_12029 = set(get_df_12009("삼성전자").columns)
    col_union = col_12003 | col_12021 | col_12023 | col_12029
    assert col_union.intersection(set(get_stock_pack("삼성전자").columns))


def test_get_df_12009():
    cols = [
        "거래량_기관",
        "거래량_기타법인",
        "거래량_개인",
        "거래량_외국인",
        "거래량_전체",
        "거래대금_기관",
        "거래대금_기타법인",
        "거래대금_개인",
        "거래대금_외국인",
        "거래대금_전체",
    ]
    assert set(cols).intersection(set(get_df_12009("삼성전자").columns))


def test_get_adjusted_price():
    df = pd.DataFrame(
        {
            "종가": [50000, 11000],
            "거래량": [1000, 6000],
            "상장주식수": [100, 500],
        },
        index=["2021-05-21", "2021-05-20"],
    )
    assert get_adjusted_price(df, "종가")[1] == 55000
    assert get_adjusted_price(df, "거래량")[1] == 1200
    assert get_adjusted_price(df, "종가", inplace=True)[1] == df.수정종가[1]



import os

from coredotfinance.krx.core import webio


def get_jsp_soup(mdcstat):
    """
    jsp_soup will be used for getting korean columns, valid_post_params.

    Parameters
    ----------
    mdcstat : str

    Returns
    -------
    jsp_soup : bs4.BeautifulSoup

    """
    jsp_filename = mdcstat[:-2]
    if _is_file(jsp_filename):
        jsp_soup = webio.soup(_load_jsp(jsp_filename))
    else:
        url = f"http://data.krx.co.kr/contents/MDC/STAT/standard/{jsp_filename}.jsp"
        jsp_soup = webio.get(url)
        _save_jsp(jsp_filename, jsp_soup)
    return jsp_soup


"""
자주 사용하는 기능의 jsp 파일은 저장해서 불러오는 걸로 사용하자.
server에 2번 요청하지 않도록.
"""

FILE_PATH = os.path.dirname(os.path.realpath(__file__))


def _is_file(jsp_filename):
    path = os.path.join(FILE_PATH, f'jsp/{jsp_filename}.jsp')
    return os.path.isfile(path)


def _save_jsp(jsp_filename, jsp_soup):
    path = os.path.join(FILE_PATH, f'jsp/{jsp_filename}.jsp')
    with open(path, 'w', encoding='utf-8') as f:
        f.write(str(jsp_soup))


def _load_jsp(jsp_filename):
    path = os.path.join(FILE_PATH, f'jsp/{jsp_filename}.jsp')
    with open(path, 'r', encoding='utf-8') as f:
        return f.read()



# -*- coding: utf-8 -*-
from coredotfinance.krx.core.krx_website.info import Info


class Product(Info):
    def __init__(
        self, code, symbol, start, end, date, **kwargs
    ):
        """
        증권상품
        :param code:
        :param start:
        :param end:
        :param day:
        :param product:
        :param code_to_function:
        """
        self.code = code
        self.start = start
        self.end = end
        self.date = date
        self.symbol = symbol

        if symbol:
            self.data_nm, self.data_cd, self.data_tp = self.autocomplete(symbol, 'product')
        self.search_type = kwargs.get("search_type", None)
        self.trade_index = kwargs.get("trade_index", None)
        self.trade_check = kwargs.get("trade_check", None)
        self.quarter = kwargs.get("quarter", None)
        self.issuing = kwargs.get("issuing", None)
        self.basic_asset = kwargs.get("basic_asset", None)

        code_to_function = {
            "13101": self.etf_price_of_entire_items,
            "13102": self.etf_fluc_of_entire_items,
            "13103": self.etf_price_trend_of_item,
            "13104": self.etf_info_of_entire_items,
            "13105": self.etf_info_of_item,
            "13106": self.etf_trade_performance_per_investor,
            "13107": self.etf_trade_performance_per_investor_item,
            "13108": self.etf_portfolio_deposit_file,
            "13109": self.detail_of_ETF,
            "13110": self.result_of_active_ETF,
            "13111": "not now",
            "13112": self.etf_trend_of_tracking_error,
            "13113": self.etf_trend_of_differential,
            "13114": self.etf_trend_of_closing_differential,
            "13115": self.risk_of_multi_ETF_trader,
            "13116": self.managing_index_and_security_of_multi_ETF,
            "13117": self.etf_assessment_of_LP_per_quarter,
            "13201": self.etn_price_of_entire_items,
            "13202": self.etn_fluc_of_entire_items,
            "13203": self.etn_price_trend_of_item,
            "13204": self.etn_info_of_entire_items,
            "13205": self.etn_info_of_item,
            "13206": self.etn_trade_performance_per_investor,
            "13207": self.etn_trade_performance_per_investor_item,
            "13208": self.detail_of_ETN,
            "13209": self.etn_product_consisting_of_index,
            "13210": self.etn_credit_and_NCR,
            "13211": self.etn_risk_of_credit,
            "13212": self.condition_of_early_repayment_loss_limited_ETN,
            "13213": self.consideration_of_range_accurual_of_loss_limited_ETN,
            "13214": self.etn_trend_of_differential,
            "13215": self.etn_trend_of_closing_differential,
            "13301": self.elw_price_of_entire_items,
            "13302": self.elw_trend_of_item,
            "13303": self.elw_info_of_entire_items,
            "13304": self.elw_entire_info_of_item,
            "13305": self.elw_trade_performance_per_investor,
            "13306": self.elw_trade_performance_per_basic_asset,
            "13307": self.elw_item_of_residual_expiration_status,
            "13308": self.elw_item_of_beneficial_expiration_status,
            "13309": self.elw_approach_level_of_early_close,
            "13310": self.elw_log_of_early_close,
            "13311": self.elw_status_of_floating_per_basic_asset,
            "13312": self.elw_status_of_floating_per_issuing,
            "13313": self.elw_assessment_of_LP_per_quarter,
        }

        self.get_requested_data = code_to_function[code]

    def etf_price_of_entire_items(self):
        """
        전종목 시세[13101]
        :arg
            day
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT04301",
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def etf_fluc_of_entire_items(self):
        """
        전종목 등락률[13102]
        :arg
            start, end
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT04401",
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def etf_price_trend_of_item(self):
        """
        개별종목 시세 추이[13103]
        :arg
            product, start, end
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT04501",
            "tboxisuCd_finder_secuprodisu1_31": f"{self.data_tp}/{self.data_nm}",
            "isuCd": self.data_cd,
            "isuCd2": self.data_cd,
            "codeNmisuCd_finder_secuprodisu1_31": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def etf_info_of_entire_items(self):
        """전종목 기본정보[13104]"""
        data = {"bld": "dbms/MDC/STAT/standard/MDCSTAT04601", "share": 1}
        return self.update_requested_data(data)

    def etf_info_of_item(self):
        """개별종목 종합정보[13105]"""
        """Not now"""
        pass

    def etf_trade_performance_per_investor(self):
        """
        투자자별 거래실적 [13106]
        :arg
            search_type, trade_index, trade, start, end
        """
        if self.search_type == "일별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT04802"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT04801"
        data = {
            "bld": bld,
            "inqTpCd": self.search_type,
            "inqCondTpCd1": self.trade_index,
            "inqCondTpCd2": self.trade_check,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def etf_trade_performance_per_investor_item(self):
        """
        투자자별 거래실적(개별종목) [13107]
        :arg
            search_type, trade_index, trade, product, start, end
        """
        if self.search_type == "일별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT04902"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT04901"
        data = {
            "bld": bld,
            "inqTpCd": self.search_type,
            "inqCondTpCd1": self.trade_index,
            "inqCondTpCd2": self.trade_check,
            "tboxisuCd_finder_secuprodisu1_4": f"{self.data_tp}/{self.data_nm}",
            "isuCd": self.data_cd,
            "isuCd2": self.data_cd,
            "codeNmisuCd_finder_secuprodisu1_4": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def etf_portfolio_deposit_file(self):
        """
        PDF (Porrfolio Deposit File) [13108]
        :arg
            product, day
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT05001",
            "tboxisuCd_finder_secuprodisu1_6": f"{self.data_tp}/{self.data_nm}",
            "isuCd": self.data_cd,
            "isuCd2": self.data_cd,
            "codeNmisuCd_finder_secuprodisu1_6": self.data_nm,
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def detail_of_ETF(self):
        """
        ETF 상세검색 [13109]
        Not now
        """
        pass

    def result_of_active_ETF(self):
        """
        액티브 ETF 실적 [13110]
        search_type, product, start, end
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT05701",
            "cls": self.search_type,
            "isuCd": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def incorporated_asset_of_active_ETF(self):
        """
        액티브ETF 편입자산현황 [13111]
        :arg
            day, product, search_type
        """
        if self.search_type == "잔존만기":
            bld = "dbms/MDC/STAT/standard/MDCSTAT05801"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT05802"
        year = self.date[:4]
        month = self.date[4:6]
        data = {
            "bld": bld,
            "isuCd": self.data_nm,
            "startYear": year,
            "startMonth": month,
        }
        return self.update_requested_data(data)

    def etf_trend_of_tracking_error(self):
        """
        추적오차율 추이 [13112]
        :arg
            product, start, end
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT05901",
            "isuCd": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def etf_trend_of_differential(self):
        """
        괴리율 추이 [13113]
        :arg
            product, start, end
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT06001",
            "isuCd": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def etf_trend_of_closing_differential(self):
        """
        장마감 괴리율 추이 [13114]
        :arg
            product, start, end
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT17101",
            "isuCd": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }

        return self.update_requested_data(data)

    def risk_of_multi_ETF_trader(self):
        """
        합성ETF 거래상대방 위험 [13115]
        :arg
            product, search_type
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT06101",
            "cls": self.search_type,
            "isuCd": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def managing_index_and_security_of_multi_ETF(self):
        """
        합성ETF 기초지수 및 담보관리[13116]
        """
        data = {"bld": "dbms/MDC/STAT/standard/MDCSTAT06201"}
        return self.update_requested_data(data)

    def etf_assessment_of_LP_per_quarter(self):
        """분기별 LP 평가[]13117"""
        year = self.date[:4]
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT06301",
            "year": year,
            "quarter": self.quarter,
        }
        return self.update_requested_data(data)



    def etn_price_of_entire_items(self):
        """
        전종목 시세 [13201]
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT06401",
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def etn_fluc_of_entire_items(self):
        """
        전종목 등락률 [13202]
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT06501",
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def etn_price_trend_of_item(self):
        """
        개별종목 시세추이 [13203]
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT06601",
            "tboxisuCd_finder_secuprodisu2_5": f"{self.data_tp}/{self.data_nm}",
            "isuCd": self.data_cd,
            "codeNmisuCd_finder_secuprodisu2_5": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def etn_info_of_entire_items(self):
        """
        전종목 기본정보 [13204]
        """
        data = {"bld": "dbms/MDC/STAT/standard/MDCSTAT06701"}
        return self.update_requested_data(data)

    def etn_info_of_item(self):
        """
        개별종목 종합정보 [13205]
        not now
        """
        pass

    def etn_trade_performance_per_investor(self):
        """
        투자자별 거래실적 [13206]
        """
        if self.search_type == "일별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT06902"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT06901"

        data = {
            "bld": bld,
            "inqTpCd": self.search_type,
            "inqCondTpCd1": self.trade_index,
            "inqCondTpCd2": self.trade_check,
            "strtDd": self.start,
            "endDd": self.end,
        }

        return self.update_requested_data(data)

    def etn_trade_performance_per_investor_item(self):
        """
        투자자별 거래실적(개별종목) [13207]
        """
        if self.search_type == "일별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT07002"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT07001"

        data = {
            "bld": bld,
            "inqTpCd": self.search_type,
            "inqCondTpCd1": self.trade_index,
            "inqCondTpCd2": self.trade_check,
            "tboxisuCd_finder_secuprodisu2_10": f"{self.data_tp}/{self.data_nm}",
            "isuCd": self.data_cd,
            "codeNmisuCd_finder_secuprodisu2_10": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def detail_of_ETN(self):
        """
        ETN 상세검색 [13208]
        not now
        """
        pass

    def etn_product_consisting_of_index(self):
        """
        기초지수 구성요소 [13209]
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT07601",
            "tboxisuCd_finder_secuprodisu2_2": self.data_nm,
            "isuCd": self.data_cd,
            "isuCd2": self.data_tp,
            "codeNmisuCd_finder_secuprodisu2_2": self.data_nm,
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def etn_credit_and_NCR(self):
        """
        발행사 신용등급 및 NCR [13210]
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT07701",
            "tboxisuCd_finder_secuprodisu2_3": f"{self.data_tp}/{self.data_nm}",
            "isuCd": self.data_cd,
            "codeNmisuCd_finder_secuprodisu2_3": self.data_nm,
        }
        return self.update_requested_data(data)

    def etn_risk_of_credit(self):
        """
        발행사 신용위험지표 [13211]
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT07801",
            "isurCd": self.issuing,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def condition_of_early_repayment_loss_limited_ETN(self):
        """
        손실제한ETN 조기상환 조건 [13212]
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT07901",
            "isuCd": self.data_cd,
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def consideration_of_range_accurual_of_loss_limited_ETN(self):
        """
        손실제한ETN 레인지어쿠루얼 참고사항 [13213]
        No data!!
        """
        pass

    def etn_trend_of_differential(self):
        """
        괴리율 추이 [13214]
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT08101",
            "isuCd": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def etn_trend_of_closing_differential(self):
        """
        장마감 괴리율 추이 [13215]
        """
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT17201",
            "isuCd": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def etn_assessment_of_LP_per_quarter(self):
        """분기별 LP 평가 [13216]"""
        """Web page error """

    def elw_price_of_entire_items(self):
        """전종목 시세[13301]"""
        data = {"bld": "dbms/MDC/STAT/standard/MDCSTAT08301", "trdDd": self.date}
        return self.update_requested_data(data)

    def elw_trend_of_item(self):
        """개별종목 시세 추이[13302]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT08401",
            "tboxisuCd_finder_secuprodisu3_34": f"{self.data_tp}/{self.data_nm}",
            "isuCd": self.data_cd,
            "isuCd2": self.data_tp,
            "codeNmisuCd_finder_secuprodisu3_34": self.data_nm,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def elw_info_of_entire_items(self):
        """전종목 기본정보[13303]"""
        data = {"bld": "dbms/MDC/STAT/standard/MDCSTAT08501"}
        return self.update_requested_data(data)

    def elw_entire_info_of_item(self):
        """개별종목 종합정보[13304]
        Not now"""
        pass

    def elw_trade_performance_per_investor(self):
        """투자자별 거래실적[13305]"""
        if self.search_type == "일별추이":
            bld = "dbms/MDC/STAT/standard/MDCSTAT08702"
        elif self.search_type == "상세보기":
            bld = "dbms/MDC/STAT/standard/MDCSTAT08703"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT08701"

        data = {
            "bld": bld,
            "inqTpCd": self.search_type,
            "inqCondTpCd1": self.trade_index,
            "inqCondTpCd2": self.trade_check,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def elw_trade_performance_per_basic_asset(self):
        """기초자산별 거래실적[13306]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT08801",
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def elw_item_of_residual_expiration_status(self):
        """개별종목 잔존만기현황[13307]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT08901",
            "elwEoTpCd": self.search_type,
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def elw_item_of_beneficial_expiration_status(self):
        """개별종목 만기손익현황[13308]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT09001",
            "elwEoTpCd": self.search_type,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def elw_approach_level_of_early_close(self):
        """조기종료 접근[13309]"""
        data = {"bld": "dbms/MDC/STAT/standard/MDCSTAT09201", "trdDd": self.date}
        return self.update_requested_data(data)

    def elw_log_of_early_close(self):
        """조기종료 발생내역[13310]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT09301",
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def elw_status_of_floating_per_basic_asset(self):
        """기초자산별 상장현황[13311]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT09401",
            "elwRghtTpKindCd": self.search_type,
            "elwUlyTpCd": self.basic_asset,
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def elw_status_of_floating_per_issuing(self):
        """발행사별 상장현황[13312]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT09501",
            "elwEoTpCd": self.search_type,
            "trdDd": self.date,
        }
        return self.update_requested_data(data)

    def elw_assessment_of_LP_per_quarter(self):
        """분기별 LP 평가 [13313]"""
        """Web page error """
        pass



import pandas as pd
import requests

from coredotfinance.util import datetime_util
from coredotfinance.util import dataframe_util


def request_get_data(symbol, start_timestamp, end_timestamp):
    """Yahoo Finance의 symbol의 History 조회"""
    url = f"https://query2.finance.yahoo.com/v8/finance/chart/{symbol}"
    headers = {
        "Content-Type": "application/json",
        "User-Agent": "Mozilla/5.0",
        "referer": "http://m.naver.com",
    }
    params = {
        "includeAdjustedClose": "true",
        "interval": "1d",
        "period1": start_timestamp,
        "period2": end_timestamp,
    }
    response = requests.get(url, headers=headers, params=params)
    return response.json()


def get_ohlcv(
    symbol, *, start=None, end=None, adjust_price=True, real_price=False
) -> pd.DataFrame:
    """Yahoo Finance의 symbol를 활용하여 가격정보(OHLCV) 조회"""
    if start is None:
        start = "19000101"
    if end is None:
        end = datetime_util.get_date_today()

    start_timestamp = datetime_util.convert_date2timestamp_sec(start)
    end_timestamp = datetime_util.convert_date2timestamp_sec(end)
    response = request_get_data(symbol, start_timestamp, end_timestamp)

    timestamp = response["chart"]["result"][0]["timestamp"]
    datetime = datetime_util.convert_timestamp2datetime_list(timestamp)
    open = response["chart"]["result"][0]["indicators"]["quote"][0]["open"]
    high = response["chart"]["result"][0]["indicators"]["quote"][0]["high"]
    low = response["chart"]["result"][0]["indicators"]["quote"][0]["low"]
    close = response["chart"]["result"][0]["indicators"]["quote"][0]["close"]
    adjclose = response["chart"]["result"][0]["indicators"]["adjclose"][0]["adjclose"]
    volume = response["chart"]["result"][0]["indicators"]["quote"][0]["volume"]
    df = pd.DataFrame(
        {
            "datetime": datetime,
            "open": open,
            "high": high,
            "low": low,
            "close": close,
            "adj_close": adjclose,
            "volume": volume,
        },
    )

    if adjust_price:
        df = apply_adjust_price(df)
    elif real_price:
        df = apply_real_price(df)

    df = dataframe_util.rename_cols2kor(df)
    df = dataframe_util.set_index_datetime(df)
    return df


def apply_adjust_price(data: pd.DataFrame) -> pd.DataFrame:
    """Yahoo Finance의 수정종가를 활용하여 다른 수정 가격"""
    df = data.copy()
    ratio = df["close"] / df["adj_close"]
    df["adj_open"] = df["open"] / ratio
    df["adj_high"] = df["high"] / ratio
    df["ajd_low"] = df["low"] / ratio
    df["ajd_volume"] = df["volume"] * ratio

    df = df.drop(["open", "high", "low", "close", "volume"], axis=1)

    df = df.rename(
        columns={
            "adj_open": "open",
            "adj_high": "high",
            "ajd_low": "low",
            "adj_close": "close",
            "ajd_volume": "volume",
        },
    )

    df = df[["open", "high", "low", "close", "volume"]]
    return df[["open", "high", "low", "close", "volume"]]


def apply_real_price(data: pd.DataFrame) -> pd.DataFrame:
    df = data.copy()
    df = df.drop(["adj_close"], axis=1)

    df = df[["open", "high", "low", "close", "volume"]]
    return df[["open", "high", "low", "close", "volume"]]



from coredotfinance.krx.core.krx_website import stock
from coredotfinance.krx.core.krx_website import products
from coredotfinance.krx.core.function_code_list import *


def get_krx_instance(code, symbol, start, end, date, **kwargs):
    # Fixme : 지금은 stock, product만 추가되어 있다.
    #   index, commodity, bond, derivatice, oversees 등도 추가해주자.

    day = kwargs.get('day')
    division = kwargs.get('division')
    item = kwargs.get('item')

    if code in stock_code_list:
        krx_instance = stock.Stock(code, symbol=symbol, start=start, end=end, date=date, **kwargs)
    elif code in product_code_list:
        krx_instance = products.Product(code, symbol=symbol, start=start, end=end, date=date, **kwargs)


    else:
        raise ValueError(f"No function code, [{code}]")

    return krx_instance



from coredotfinance.krx.core import webio


class Info:
    def __init__(self, start, end, date):
        """지수
        :param start: 시작일
        :param end: 종료일
        :param date: 기준일
        """

        self.start = start
        self.end = end
        self.day = date

    def update_requested_data(self, requested_data):
        requested_data["MIME Type"] = "application/x-www-form-urlencoded; charset=UTF-8"
        requested_data["csvxls_isNo"] = "false"
        return requested_data

    def autocomplete(self, symbol, kind):
        """
        Returns data_nm, data_cd, data_tp from below result of requests.

        Parameters
        ----------
        symbol : str
            symbol user inputs.
        kind : str
            by the type, data

        See Also
        -------
        below is the result of requests.post(url, data)
        html.content :
        b'{
            "block1": [
                {
                    "full_code": "KR7006401004",
                    "short_code": "006405",
                    "codeName": "\xec\x82\xbc\xec\x84\xb1SDI\xec\x9a\xb0",
                    "marketCode": "STK",
                    "marketName": "\xec\x9c\xa0\xea\xb0\x80\xec\xa6\x9d\xea\xb6\x8c",
                    "marketEngName": "KOSPI",
                    "ord1": "999",
                    "ord2": "16",
                }
            ],
            "CURRENT_DATETIME": "2021.06.18 PM 06:15:00",
        }'

        eval(html.content) :
            "block1": [
                {
                    "full_code": "KR7006401004",
                    "short_code": "006405",
                    "codeName": "삼성SDI우",
                    "marketCode": "STK",
                    "marketName": "유가증권",
                    "marketEngName": "KOSPI",
                    "ord1": "999",
                    "ord2": "16",
                }
            ],
            "CURRENT_DATETIME": "2021.06.18 PM 06:15:00",
        }

        Error
        -----
        When something goes wrong, html.content looks like below
        html.content:
            b'{"block1":[],"CURRENT_DATETIME":"2021.06.22 AM 10:21:54"}'
        """
        url = "http://data.krx.co.kr/comm/bldAttendant/getJsonData.cmd"
        post_data = {
            "stock": {
                "mktsel": "ALL",
                "searchText": symbol,
                "bld": "dbms/comm/finder/finder_stkisu",
            },
            "product": {
                "mktsel": "ALL",
                "searchText": symbol,
                "bld": "dbms/comm/finder/finder_secuprodisu",
            },
        }
        html = webio.post(url=url, data=post_data[kind])
        html_list = eval(html.content)["block1"]
        if not html_list:
            raise ValueError(f"Can not find any data from krx. symbol : {symbol}, kind : {kind}")
        else:
            html_dict = html_list[0]
            return html_dict["codeName"], html_dict["full_code"], html_dict["short_code"]



# -*- coding: utf-8 -*-
from coredotfinance.krx.core.krx_website.info import Info


class Oversees(Info):
    def __init__(self, code, start, end, day, code_to_function):
        super().__init__(start, end, day)
        self.get_requested_data = code_to_function[code]


class EUREX(Oversees):
    def __init__(self, code, start, end, day, **kwargs):
        code_to_function = {
            "17101": "Requested JSON parse failed. [Failed]",
            "17102": "No data",
            "17103": "No data",
            "17104": self.trend_of_trade_perform_per_right_type,
            "17105": self.trend_of_trade_perform_per_right_type_monthly,
            "17106": self.price_table_per_expire,
            "17107": "No data",
            "17108": self.price_trend_of_EURO_STOXX,
        }
        self.right_type = kwargs.get("right_type", None)
        self.inquiry = kwargs.get("inquiry", None)
        super(EUREX, self).__init__(code, start, end, day, code_to_function)

    def trend_of_trade_perform_per_right_type(self):
        """권리유형별 거래실적 추이[17104]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT16501",
            "rghtTpCd": self.right_type,
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)

    def trend_of_trade_perform_per_right_type_monthly(self):
        """권리유형별 거래실적 추이(월)[17105]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT16601",
            "rghtTpCd": self.right_type,
            "strtYy": self.start[:4],
            "strtMm": self.start[4:6],
            "endYy": self.end[:4],
            "endMm": self.end[4:6],
        }
        return self.update_requested_data(data)

    def price_table_per_expire(self):
        """만기별 가격표(옵션)[17106]"""
        if self.inquiry == "총거래량":
            bld = "dbms/MDC/STAT/standard/MDCSTAT16802"
        else:
            bld = "dbms/MDC/STAT/standard/MDCSTAT16801"

        data = {"bld": bld, "trdDd": self.day[:6]}
        return self.update_requested_data(data)

    def price_trend_of_EURO_STOXX(self):
        """EURO STOXX 50 시세 추이[17108]"""
        data = {
            "bld": "dbms/MDC/STAT/standard/MDCSTAT16701",
            "strtDd": self.start,
            "endDd": self.end,
        }
        return self.update_requested_data(data)


class TokyoExchange(Oversees):
    pass  # No data



import pytest
from datetime import datetime
import pandas as pd
import numpy as np

from coredotfinance.krx.core import process


@pytest.fixture
def example_data_json():
    return {
        "output": [
            {
                "TRD_DD": "2021/04/26",
                "TDD_CLSPRC": "375,500",
                "FLUC_TP_CD": "2",
                "CMPPREVDD_PRC": "-2,500",
                "FLUC_RT": "-0.66",
                "TDD_OPNPRC": "376,500",
                "TDD_HGPRC": "379,000",
                "TDD_LWPRC": "375,500",
                "ACC_TRDVOL": "167,042",
                "ACC_TRDVAL": "62,964,265,000",
                "MKTCAP": "61,680,904,822,500",
                "LIST_SHRS": "164,263,395",
            }
        ],
        "CURRENT_DATETIME": "2021.04.26 AM 10:57:43",
    }


@pytest.fixture
def example_column_map():
    return {
        "TRD_DD": "일자",
        "TDD_CLSPRC": "종가",
        "CMPPREVDD_PRC": "대비",
        "FLUC_RT": "등락률",
        "TDD_OPNPRC": "시가",
        "TDD_HGPRC": "고가",
        "TDD_LWPRC": "저가",
        "ACC_TRDVOL": "거래량",
        "ACC_TRDVAL": "거래대금",
        "MKTCAP": "시가총액",
        "LIST_SHRS": "상장주식수",
        "BND_CLSS_NM1": "구분1",
        "BND_CLSS_NM2": "구분2",
    }


def test_check_data_validation(example_data_json):
    try:
        process.check_data_validation(example_data_json)
        test = True
    except Exception:
        test = False
    assert test


def test_check_data_validation_wrong():
    no_data = {"output": [], "CURRENT_DATETIME": "2021.04.26 AM 10:57:43"}
    try:
        process.check_data_validation(no_data)
        test = False
    except Exception:
        test = True
    assert test


def test_apply_column_map(example_data_json, example_column_map):
    test = list(process.apply_column_map(example_data_json, example_column_map))
    answer = ["일자", "종가", "대비", "등락률", "시가", "고가", "저가", "거래량", "거래대금", "시가총액", "상장주식수"]
    assert test == answer


def test_date_to_index():
    test_data = pd.DataFrame([["2021/04/21", "100"]], columns=["일자", "주가"])
    assert process.date_to_index(test_data).index[0] == datetime(2021, 4, 21)


def test_date_to_index_wrong():
    test_data = pd.DataFrame(
        [["2021/04/21", "100"], ["2021/04/21", "120"]], columns=["일자", "주가"]
    )
    assert process.date_to_index(test_data).index[0] == 0


def test_remove_same_named_column():
    test_data_1 = [["종가"], ["대비"], ["등락률", "주가"]]
    test_data_2 = [["종가", ""], ["대비", ""], ["등락률", "주가"]]
    assert process.remove_same_named_column(test_data_1, 2) == test_data_2


def test_multi_columnize():
    test_data_1 = [["종가", ""], ["대비", ""], ["등락률", "주가"]]
    test_data_2 = [["종가", "대비", "등락률"], ["", "", "주가"]]
    assert process.multi_columnize(test_data_1, 2) == test_data_2


def test_string_to_float():
    test_data = pd.DataFrame(
        [["100,000", "삼성", "-", "1000", "1000.33", 1, 1000.33]],
        columns=["주가", "str", "-", "int_str", "float_str", "int", "float"],
    )
    assert list(process.string_to_float(test_data).loc[0]) == [
        100000.0,
        "삼성",
        np.nan,
        1000.0,
        1000.33,
        1,
        1000.33,
    ]
